/*====================================+=====================================+
! File matelefree_info.h              ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+---------------------------------------------------------------------------+
!                                                                           !
!                        Global text information                            !
!                                                                           !
+-------+-------------------------------------------------------------------+
! Notes !                                                                   !
+-------+                                                                   !
! Suffixes :                                                                !
!    _8859  : used in MSW ".rc"                                             !
!    _ASCII : used in command line usage                                    !
!                                                                           !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include "matelefree_build_number.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
#define MTF_DISPLAY_NAME_ASCII  "MaTeleFree"
#define MTF_DISPLAY_NAME_UTF8   u8"MaT\u00e9l\u00e9Free"
#define MTF_DISPLAY_NAME        wxString::FromUTF8( MTF_DISPLAY_NAME_UTF8 )
#define MTF_DISPLAY_NAME_8859   "MaT\Xe9l\Xe9Free"
#define MTF_NAME                "matelefree"
/*--( Under GTK, "http://" generates 2 url open calls )--------------------*/
#define MTF_HOST          "scarabee-software.net"
#define MTF_URL_SITE      MTF_HOST
#define MTF_URL_FORUM     MTF_HOST "/forum"
#define MTF_URL_DONATION  "https://www.paypal.com/cgi-bin/webscr?" \
                            "cmd=_s-xclick&hosted_button_id=BHASPCZZFZU62"
#define MTF_URL_VERSION \
        "http://" MTF_HOST "/download/matelefree/matelefree_version.txt"
/*-------------------------------------------------------------------------*/
#ifdef __x86_64__
   #define MTF_CPU_TYPE     "x86_64"
   #define MTF_BITNESS_NUM  64
   #define MTF_BITNESS_TEXT wxTRANSLATE( "64-bit" )
#elif defined( __i386__ )
   #define MTF_CPU_TYPE     "i386"
   #define MTF_BITNESS_NUM  32
   #define MTF_BITNESS_TEXT wxTRANSLATE( "32-bit" )
#elif defined( __arm__ )
   #define MTF_CPU_TYPE     "armhf"
   #define MTF_BITNESS_NUM  32
   #define MTF_BITNESS_TEXT wxTRANSLATE( "32-bit" )
#else
   #error Unknown platform to: define cpu type and bitness
#endif
/*-------------------------------------------------------------------------*/
#ifdef __WXMSW__
   #define MTF_OS_TYPE "windows"
#elif defined( __WXGTK__ )
   #define MTF_OS_TYPE "gnulinux"
#else
   #error Unknown platform to: define MTF_OS_TYPE
#endif
/*-------------------------------------------------------------------------*/
#define MTF_BUILT_TYPE  MTF_OS_TYPE "_" MTF_CPU_TYPE
/*-------------------------------------------------------------------------*/
#define MTF_VERSION       "1.23" //" beta 1 (" MTF_BUILD_NUMBER ")"
#define MTF_VERSION_MSWRC  1,23,0,0
/*-------------------------------------------------------------------------*/
#define MTF_DATES          "2016-2018"
#define MTF_COMPANY_ASCII  "Remi PASCAL"
#define MTF_COMPANY_UTF8   u8"R\u00e9mi PASCAL"
#define MTF_COMPANY        wxString::FromUTF8( MTF_COMPANY_UTF8 )
#define MTF_COMPANY_8859   "R\Xe9mi PASCAL"
/*-------------------------------------------------------------------------*/
#define MTF_COPYRIGHT_ASCII \
        "Copyright (C) " MTF_DATES " " MTF_COMPANY_ASCII
#define MTF_COPYRIGHT_8859 \
        "Copyright \Xa9 " MTF_DATES " " MTF_COMPANY_8859
#define MTF_COPYRIGHT \
          wxString::FromUTF8( u8"Copyright \u00a9" ) \
        + " " MTF_DATES " " + MTF_COMPANY
/*-------------------------------------------------------------------------*/
#define MTF_DESCRIPTION_8859 \
        wxTRANSLATE( "MaT\Xe9l\Xe9\X46ree : a multimedia stream player "\
                     "using VLC's libvlc" \
                   )
#define MTF_DESCRIPTION \
        wxString::FromUTF8( wxTRANSLATE( u8"MaT\u00e9l\u00e9Free : " \
                                         "a multimedia stream player " \
                                         "using VLC's libvlc" \
                                       ) \
                          )
/*-------------------------------------------------------------------------*/
#define MTF_LICENSE_ASCII \
        wxTRANSLATE( "MaTeleFree is free of charge and " \
                     "distributed under the GPLv3 license" \
                   )
#define MTF_LICENSE \
        wxString::FromUTF8( wxTRANSLATE( u8"MaT\u00e9l\u00e9Free is free " \
                                         "of charge and distributed under " \
                                         "the GPLv3 license" \
                                       ) \
                          )

/*--( Under GTK, "http://" generates 2 url open calls )--------------------*/
#define MTF_LICENSE_URL "www.gnu.org/licenses/gpl.html"

/*-------------------------------------------------------------------------*/
#define MTF_MAIL \
        wxString::FromUTF8( u8"MaT\u00e9l\u00e9Free " \
                            "<matelefree@scarabee-software.net>" \
                          )
#define MTF_MAIL_TRANSLATOR \
        "English <matelefree.translation@scarabee-software.net>"
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file matelefree_info.h                      !
+==========================================================================*/
