/*====================================+=====================================+
! File CFrame.cpp                     ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/filename.h>
#include <wx/dir.h>
#include <wx/srchctrl.h>
#include <wx/aui/framemanager.h>
#include <wx/aui/dockart.h>
#include <wx/treectrl.h>
#include <wx/tokenzr.h>
#include <wx/statbmp.h>
#include <wx/slider.h>
#include <wx/wupdlock.h>
#include <wx/busyinfo.h>
/*-------------------------------------------------------------------------*/
#include "matelefree.h"
#include "matelefree_info.h"
#include "CFrame.h"
#include "CApp.h"
#include "CDlgVersion.h"
#include "CDlgPrefs.h"
#include "CDlgPlName.h"
#include "CDlgAbout.h"
#include "CDownload.h"
/*-------------------------------------------------------------------------*/
#include "tango/media_playback_start_32x32.xpm"
#include "tango/media_playback_start_16x16.xpm"
#include "tango/media_playback_pause_16x16.xpm"
#include "tango/media_playback_pause_32x32.xpm"
#include "tango/media_playback_stop_32x32.xpm"
#include "tango/media_playback_stop_16x16.xpm"
#include "tango/media_skip_backward_32x32.xpm"
#include "tango/media_skip_backward_16x16.xpm"
#include "tango/media_skip_forward_32x32.xpm"
#include "tango/media_skip_forward_16x16.xpm"
#include "tango/image_loading_32x32.xpm"
#include "tango/image_loading_16x16.xpm"
#include "tango/camera_photo_32x32.xpm"
#include "tango/camera_photo_16x16.xpm"
#include "tango/audio_volume_muted_32x32.xpm"
#include "tango/audio_volume_muted_16x16.xpm"
#include "tango/audio_volume_medium_32x32.xpm"
#include "tango/audio_volume_medium_16x16.xpm"
#include "tango/audio_volume_high_16x16.xpm"
#include "tango/audio_volume_low_16x16.xpm"
#include "tango/preferences_desktop_16x16.xpm"
#include "tango/software_update_available_16x16.xpm"
#include "tango/system_users_16x16.xpm"
#include "tango/internet_web_browser_16x16.xpm"
#include "tango/text_x_generic_template_32x32.xpm"
#include "tango/text_x_generic_template_16x16.xpm"
/*-------------------------------------------------------------------------*/
#ifndef __WXMSW__
#include "matelefree.xpm"
#endif // __WXMSW__
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CTaskBarIcon::CTaskBarIcon() : wxTaskBarIcon()
{
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_TASKBAR_LEFT_DOWN, &CTaskBarIcon::OnLeftDown, this ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CTaskBarIcon::OnLeftDown( wxTaskBarIconEvent &event )
{
   /*----------------------------------------------------------------------*/
   CFrame    *frame   = wxGetApp().m_frame   ;
   wxMenuBar *menubar = frame->get_menubar() ;
   wxMenu    menu ;
   /*----------------------------------------------------------------------*/
   menu.AppendCheckItem( ID_TOGGLE_SHOW, _( "&Show main window" ),
                         _( "Show or hide the main window" )
                       )->Check( frame->IsShown() ) ;
   /*----------------------------------------------------------------------*/
   if(    wxGetApp().M_boo_mini_display.get()
       && frame->get_mini_display_smallest()
     )
   {  /*-------------------------------------------------------------------*/
      menu.AppendCheckItem( ID_TOGGLE_CAPTION,
                      _( "Show window &caption (title bar)" ),
                      _( "Show or hide main the window caption (title bar)" )
                     )->Check( wxGetApp().M_boo_mini_display_caption.get() );
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   menu.AppendSeparator() ;
   mtf::append_item( &menu, menubar, ID_TOGGLE_MINI_DISPLAY ) ;
   menu.AppendSeparator() ;
   mtf::append_item( &menu, menubar, ID_PLAY_START_PAUSE ) ;
   mtf::append_item( &menu, menubar, ID_PLAY_STOP ) ;
   mtf::append_item( &menu, menubar, ID_PLAY_NEXT ) ;
   mtf::append_item( &menu, menubar, ID_PLAY_PREV ) ;
   mtf::append_item( &menu, menubar, ID_PLAY_HIST ) ;
   mtf::append_item( &menu, menubar, ID_PLAY_HIST_MENU ) ;
   menu.AppendSeparator() ;
   mtf::append_item( &menu, menubar, ID_VOLUME_UP ) ;
   mtf::append_item( &menu, menubar, ID_VOLUME_DOWN ) ;
   mtf::append_item( &menu, menubar, ID_TOGGLE_MUTE ) ;
   menu.AppendSeparator() ;
   mtf::append_item( &menu, menubar, wxID_PREFERENCES ) ;
   mtf::append_item( &menu, menubar, wxID_EXIT ) ;
   /*--( Process menu the standard way )-----------------------------------*/
   int i_sel_id = frame->GetPopupMenuSelectionFromUser( menu ) ;
   if( i_sel_id == wxID_NONE ) { return ; }
   /*----------------------------------------------------------------------*/
   wxCommandEvent evt_cmd( wxEVT_MENU, i_sel_id ) ;
   frame->GetEventHandler()->ProcessEvent( evt_cmd ) ;
   /*-----------------------------------------------------------------------+
   ! @BUG wxWidgets ?                                                       !
   ! When handling a "wxID_EXIT" menu command.                              !
   ! It seems that after receiving this event through the icon menu the     !
   ! event loop has to be woken up to treat it ...                          !
   +-----------------------------------------------------------------------*/
   #ifdef __WXMSW__
      if( i_sel_id == wxID_EXIT )
      {  SetTimer( frame->GetHandle(), 0, 0, nullptr ) ; }
   #endif // __WXMSW__
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CFrame::CFrame() : wxFrame( nullptr, wxID_ANY, wxEmptyString,
                            wxDefaultPosition, wxDefaultSize,
                              wxDEFAULT_FRAME_STYLE
                            | ( wxGetApp().M_boo_stay_on_top.get()
                                ? wxSTAY_ON_TOP : 0
                              ),
                            "MATELEFREE"
                          )
{  ; }

/*-------------------------------------------------------------------------*/
CFrame::~CFrame()
{
   /*----------------------------------------------------------------------*/
   delete m_taskbar_icon ;
   /*--( Display changes are useless to be showed )------------------------*/
   wxWindowUpdateLocker no_window_updates( m_panel_media ) ;

   /*--( Last channels played restarted at startup ? )---------------------*/
   if( !wxGetApp().M_boo_startup_play.get() )
   {  wxGetApp().M_s_startup_play_list.clear() ; }
   else
   {  /*-------------------------------------------------------------------*/
      CVlcMedia *p_media = get_media_active() ;
      if( p_media != nullptr )
      {  wxGetApp().M_s_startup_play_active.set( p_media->get_channel_name()
                                               ) ;
      }
      /*-------------------------------------------------------------------*/
      wxString s_media ;
      /*-------------------------------------------------------------------*/
      for( const auto p_media : m_vec_media )
      {  /*----------------------------------------------------------------*/
         if( !s_media.empty() ) { s_media += '\n' ; }
         /*----------------------------------------------------------------*/
         s_media += p_media->get_channel_name() ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      wxGetApp().M_s_startup_play_list.set( s_media ) ;
      /*-------------------------------------------------------------------*/
   }

   /*--( The current state (mini or not) perspective is "saved" )----------*/
   save_display_state() ;
   /*----------------------------------------------------------------------*/
   wxGetApp().M_s_bopl_pl_user_list.set( m_bopl->get_pl_user_list() ) ;
   wxGetApp().M_s_bopl_expanded.set( m_bopl->get_expanded() ) ;
   wxGetApp().M_s_bopl_sel_name.set( m_bopl->get_selection_name() ) ;
   /*----------------------------------------------------------------------*/
   wxGetApp().M_s_play_hist.set( get_play_hist() ) ;
   /*----------------------------------------------------------------------*/
   wxGetApp().save_config_info() ;
   /*--( Release all allocated media )-------------------------------------*/
   for( auto p_media : m_vec_media ) { delete_media( p_media ) ; }
   /*----------------------------------------------------------------------*/
   m_auimgr.UnInit() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init()
{
   /*----------------------------------------------------------------------*/
   m_auimgr.SetManagedWindow( this ) ;
   /*-----------------------------------------------------------------------+
   ! Remove the 1 pixel border around panes. This is particularly visible   !
   ! around the the media one in full screen                                !
   +-----------------------------------------------------------------------*/
   m_auimgr.GetArtProvider()->SetMetric( wxAUI_DOCKART_PANE_BORDER_SIZE, 0 );
   /*----------------------------------------------------------------------*/
   m_auimgr.Bind( wxEVT_AUI_PANE_CLOSE,
                  [ this ]( wxAuiManagerEvent &event )
                  {  /*--( Strangely ... we have to set the "Show" )-------*/
                     toggle_pane_show( event.GetPane()->window ) ;
                     /*----------------------------------------------------*/
                  }
                ) ;
   /*----------------------------------------------------------------------*/
   #ifdef __WXMSW__
      SetIcon( wxIcon( "A_MTF_ICON" ) ) ;
   #else
      SetIcon( wxIcon( matelefree_xpm ) ) ;
   #endif // __WXMSW__

   /*----------------------------------------------------------------------*/
   if( wxTaskBarIcon::IsAvailable() )
   {  m_taskbar_icon = new CTaskBarIcon() ; }

   /*-----------------------------------------------------------------------+
   ! For now, only MSW can support the "smallest" mini display.             !
   ! For GTK it may depend on the graphic interface ... forcing it can be   !
   ! done via a command line parameter                                      !
   +-----------------------------------------------------------------------*/
   #ifndef __WXMSW__
   if( wxGetApp().get_force_minimalest() )
   #endif // __WXMSW__
   {  m_boo_mini_display_smallest = true ; }

   /*----------------------------------------------------------------------*/
   set_title() ;
   set_play_hist( wxGetApp().M_s_play_hist.get() ) ;

   /*-----------------------------------------------------------------------+
   ! Download management                                                    !
   +-----------------------------------------------------------------------*/
   Bind( EVT_DOWNLOAD,
         [ this ]( CDownloadEvent &event )
         {  /*-------------------------------------------------------------*/
            switch( event.GetId() )
            {
               /*----------------------------------------------------------*/
               case DID_VERSION :
                  check_version( event.get_err_code(), event.get_string(),
                                 false
                               ) ;
                  break ;
               /*----------------------------------------------------------*/
               default :
                  mtf::trace( wxString::Format(
                                        "id:%d err:%d progress:%d/%d <%s>",
                                        event.get_id(), event.get_err_code(),
                             static_cast< int >( event.get_current_size() ),
                             static_cast< int >( event.get_download_size() ),
                                        event.get_string()
                                              )
                             ) ;
                  break ;
               /*----------------------------------------------------------*/
            }
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      ) ;

   /*----------------------------------------------------------------------*/
   build_menubar() ;
   SetMenuBar( m_menubar ) ;

   /*-----------------------------------------------------------------------+
   ! Playlist tree                                                          !
   +-----------------------------------------------------------------------*/
   m_bopl = new CBookPlaylist( this, "bopl" ) ;

   /*----------------------------------------------------------------------*/
   wxGetApp().init_image_fn() ;
   wxGetApp().init_playlists() ;
   wxGetApp().init_channels() ;

   /*-----------------------------------------------------------------------+
   ! Media player                                                           !
   +-----------------------------------------------------------------------*/
   m_panel_media = new wxPanel( this ) ;
   m_panel_media->SetName( "media" ) ;
   /*--( Black is set because border around sizer items use this color )---*/
   m_panel_media->SetOwnBackgroundColour( *wxBLACK ) ;

   /*----------------------------------------------------------------------*/
   Bind( EVT_SET_TITLE,
         [ this ]( wxCommandEvent &event ) { set_title() ; }
       ) ;

   /*-----------------------------------------------------------------------+
   ! Under wxGTK "CVlcMedia::init_player_window" has to be done after       !
   ! "Show" (cf comment in CFrame::OnFirstIdle)                             !
   +-----------------------------------------------------------------------*/
   Bind( wxEVT_IDLE, &CFrame::OnFirstIdle, this ) ;
   /*-----------------------------------------------------------------------+
   ! When in full screen the mouse pointer appears when moved.              !
   ! We could have let it handle by "libvlc" but if it handles the mouse    !
   ! it catches the other mouse events that we want to handle :             !
   ! double click, middle click ...                                         !
   +-----------------------------------------------------------------------*/
   m_timer_mouse_off.SetOwner( this ) ;
   Bind( wxEVT_TIMER, &CFrame::OnFullScreenMouseOff,
         this, m_timer_mouse_off.GetId()
       ) ;
   /*----------------------------------------------------------------------*/
   m_timer_viewers.SetOwner( this ) ;
   Bind( wxEVT_TIMER, &CFrame::OnRefreshViewers,
         this, m_timer_viewers.GetId()
       ) ;
   /*-----------------------------------------------------------------------+
   ! Sizer Media                                                            !
   +-----------------------------------------------------------------------*/
   m_bsz_media = new wxBoxSizer( wxVERTICAL ) ;
   m_panel_media->SetSizer( m_bsz_media ) ;

   /*-----------------------------------------------------------------------+
   ! Play Control                                                           !
   ! ------------                                                           !
   ! A panel is created because :                                           !
   ! - a toolbar pane seems to takes the full width / height. But I want    !
   !   the control toolbar to be able to have a width limited to the one of !
   !   the media window                                                     !
   ! - the toolbar is added to a panel/sizer and the pane info parameters   !
   !   are set to values to simulate something near what is wanted          !
   +-----------------------------------------------------------------------*/
   m_panel_ctrl = new wxPanel( this ) ;
   m_panel_ctrl->SetName( "panel_ctrl" ) ;
   wxBoxSizer *bsz_ctrl = new wxBoxSizer( wxVERTICAL ) ;
   m_panel_ctrl->SetSizer( bsz_ctrl ) ;
   m_to_ctrl = new wxAuiToolBar( m_panel_ctrl ) ;
   bsz_ctrl->Add( m_to_ctrl, 1, wxEXPAND ) ;
   /*----------------------------------------------------------------------*/
   m_to_ctrl->SetName( "to_ctrl" ) ;
   m_to_ctrl->SetToolBitmapSize( wxSize( 32, 32 ) ) ;
   m_to_ctrl->AddTool( ID_PLAY_START_PAUSE, wxEmptyString,
                       media_playback_start_32x32_xpm,
                       m_menubar->FindItem( ID_PLAY_START_PAUSE )->GetHelp(),
                       wxITEM_NORMAL
                     ) ;
   m_to_ctrl->AddTool( ID_PLAY_STOP, wxEmptyString,
                       media_playback_stop_32x32_xpm,
                       m_menubar->FindItem( ID_PLAY_STOP )->GetHelp()
                     ) ;
   m_to_ctrl->SetToolDropDown( ID_PLAY_STOP, true ) ;
   m_to_ctrl->AddTool( ID_PLAY_PREV, wxEmptyString,
                       media_skip_backward_32x32_xpm,
                       m_menubar->FindItem( ID_PLAY_PREV )->GetHelp()
                     ) ;
   m_to_ctrl->AddTool( ID_PLAY_NEXT, wxEmptyString,
                       media_skip_forward_32x32_xpm,
                       m_menubar->FindItem( ID_PLAY_NEXT )->GetHelp()
                     ) ;
   m_to_ctrl->AddTool( ID_PLAY_HIST, wxEmptyString,
                       image_loading_32x32_xpm,
                       m_menubar->FindItem( ID_PLAY_HIST )->GetHelp()
                     ) ;
   m_to_ctrl->SetToolDropDown( ID_PLAY_HIST, true ) ;
   m_to_ctrl->AddSeparator() ;
   m_to_ctrl->AddTool( ID_TOGGLE_MINI_DISPLAY, wxEmptyString,
                       text_x_generic_template_32x32_xpm,
                       wxEmptyString, wxITEM_CHECK
                     ) ;
   m_to_ctrl->AddTool( ID_FULLSCREEN, wxEmptyString,
                       mtf::get_toolbar_bmp( wxART_FULL_SCREEN ),
                       wxEmptyString, wxITEM_CHECK
                     ) ;
   m_to_ctrl->AddTool( ID_SNAPSHOT, wxEmptyString,
                       camera_photo_32x32_xpm,
                       m_menubar->FindItem( ID_SNAPSHOT )->GetHelp()
                     ) ;
   m_to_ctrl->AddStretchSpacer() ;
   m_to_ctrl->AddTool( ID_TOGGLE_MUTE, wxEmptyString,
                       audio_volume_medium_32x32_xpm,
                       m_menubar->FindItem( ID_TOGGLE_MUTE )->GetHelp(),
                       wxITEM_CHECK
                     ) ;
   m_sld_volume = new wxSlider( m_to_ctrl, wxID_ANY,
                                wxGetApp().M_i_volume.get(), 0, co_i_volmax,
                                wxDefaultPosition, wxSize( co_i_volmax, -1 ),
                                wxSL_VALUE_LABEL
                              ) ;
   m_to_ctrl->AddControl( m_sld_volume, _( "Volume" ) ) ;
   m_to_ctrl->Realize() ;
   Bind( wxEVT_AUITOOLBAR_TOOL_DROPDOWN, &CFrame::OnBtPlayStop,
         this, ID_PLAY_STOP
       ) ;
   Bind( wxEVT_AUITOOLBAR_TOOL_DROPDOWN, &CFrame::OnBtPlayHist,
         this, ID_PLAY_HIST
       ) ;
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_COMMAND_SLIDER_UPDATED,
         [ this ]( wxCommandEvent &event )
         {  set_volume_from_slider() ; event.Skip() ; }
       ) ;
   m_sld_volume->Bind( wxEVT_LEFT_UP,
                       [ this ]( wxMouseEvent &event )
                       {  set_volume_from_slider() ; event.Skip() ; }
                     ) ;
   /*-----------------------------------------------------------------------+
   ! It is more user friendly to copy VLC mouse wheel action on the volume  !
   ! slider : wheel move direction and increment                            !
   +-----------------------------------------------------------------------*/
   m_sld_volume->Bind( wxEVT_MOUSEWHEEL, &CFrame::OnSldVolumeMouseWheel,
                       this
                     ) ;
   /*----------------------------------------------------------------------*/
   init_gui_play() ;
   init_gui_mini_display() ;
   init_gui_fullscreen() ;
   init_gui_mute() ;
   /*----------------------------------------------------------------------*/
   m_bopl->init_list_all() ;

   /*-----------------------------------------------------------------------+
   ! Position and size of the window                                        !
   +-----------------------------------------------------------------------*/
   if( wxGetApp().M_main_rec.get().GetSize() == wxDefaultSize )
   {
      /*-------------------------------------------------------------------*/
      wxRect rec_cur = wxGetApp().M_main_rec.get() ;
      const wxRect rec_scr = wxGetClientDisplayRect() ;
      /*--( Default frame size )-------------------------------------------*/
      rec_cur.width  = wxMax( 800, rec_scr.width  / 3.0 * 2.0 ) ;
      rec_cur.height = wxMax( 600, rec_scr.height / 4.0 * 3.0 ) ;
      /*-------------------------------------------------------------------*/
      rec_cur.SetPosition( wxDefaultPosition ) ;
      wxGetApp().M_main_rec.set( rec_cur ) ;
      /*-------------------------------------------------------------------*/
   }
   /*-----------------------------------------------------------------------+
   ! Position and size for mini display                                     !
   +-----------------------------------------------------------------------*/
   if( wxGetApp().M_main_mini_rec.get().GetSize() == wxDefaultSize )
   {
      /*-------------------------------------------------------------------*/
      wxRect rec_cur = wxGetApp().M_main_mini_rec.get() ;
      const wxRect rec_scr = wxGetClientDisplayRect() ;
      /*--( Default size )-------------------------------------------------*/
      rec_cur.width = 300 ;
      rec_cur.height = rec_cur.width / 4.0 * 3.0 ;
      /*--( And position )-------------------------------------------------*/
      rec_cur.x = rec_scr.width - rec_cur.width - 10 ;
      rec_cur.y = 10 ;
      /*-------------------------------------------------------------------*/
      wxGetApp().M_main_mini_rec.set( rec_cur ) ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Setting rect is necessary for the first pane setup )--------------*/
   setup_display_rect() ;

   /*-----------------------------------------------------------------------+
   ! Perspective                                                            !
   +-----------------------------------------------------------------------*/
   m_bopl->SetSize( GetClientSize().GetWidth() / 4, -1 ) ;
   m_auimgr.AddPane( m_bopl,
                     wxAuiPaneInfo().Name( m_bopl->GetName() )
                     .Caption( _( "Playlists" ) ).CaptionVisible( false )
                     .Gripper( true ).Left().Layer( 1 ).Position( 0 )
                   ) ;
   m_auimgr.AddPane( get_window_pl(),
                     wxAuiPaneInfo().Name( "pane_pl" )
                     .Caption( _( "Channels" ) ).CaptionVisible( false )
                     .Gripper( true )
                     .Left().Layer( 1 ).Position( 1 )
                   ) ;
   m_auimgr.AddPane( m_panel_media, wxAuiPaneInfo()
                     .Name( m_panel_media->GetName() )
                     .CenterPane()
                   ) ;
   /*--( For details check comment written for the creation )--------------*/
   wxSize size_min = m_to_ctrl->GetEffectiveMinSize() ;
   wxSize size_max = wxSize( -1, size_min.GetHeight() ) ;
   m_auimgr.AddPane( m_panel_ctrl,
                     wxAuiPaneInfo().Name( m_panel_ctrl->GetName() )
                     .Caption( _( "Play control" ) ).CaptionVisible( false )
                     .MinSize( size_min ).MaxSize( size_max )
                     .FloatingSize( size_max )
                     .LeftDockable( false ).RightDockable( false )
                     .Gripper( true ).Bottom()
                   ) ;

   /*--( Set tree nodes expand state to what it was )----------------------*/
   m_bopl->expand_list( wxGetApp().M_s_bopl_expanded.get() ) ;
   /*--( Remembered selected playlist )------------------------------------*/
   if( !wxGetApp().M_s_bopl_sel_name.empty() )
   {  m_bopl->set_selection( wxGetApp().M_s_bopl_sel_name.get() ) ; }
   /*--( Or the first one ... )--------------------------------------------*/
   if(    !m_bopl->get_selection().IsOk()
       || m_bopl->get_playlist( m_bopl->get_selection() ) == nullptr
     )
   {  m_bopl->set_selection( m_bopl->get_first_ti() ) ; }

   /*----------------------------------------------------------------------*/
   setup_display_mode() ;
   /*----------------------------------------------------------------------*/
   init_gui_pane() ;
   m_auimgr.Update() ;

   /*--( Set the floating size of panes )----------------------------------*/
   init_pane_floating_size( m_bopl ) ;
   init_pane_floating_size( get_window_pl() ) ;
   init_pane_floating_size( m_panel_ctrl ) ;

   /*--( Check new version availability )----------------------------------*/
   {
      const wxString co_s_today = wxDateTime::Today().FormatISODate() ;
      const wxString co_s_next_chkupd
               = wxGetApp().next_chkupd( wxGetApp().M_i_chkupd_freq.get() ) ;
      if( !co_s_next_chkupd.empty() && co_s_next_chkupd <= co_s_today )
      {  /*----------------------------------------------------------------*/
         CDownload dl( this, MTF_URL_VERSION ) ;
         dl.get_dlbase().set_event_id( DID_VERSION ) ;
         dl.thread_start() ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   m_bopl->make_selection_visible() ;
   /*----------------------------------------------------------------------*/
   CPlaylist *pl = get_playlist_active() ;
   if( pl != nullptr ) { pl->SetFocus() ; }
   /*----------------------------------------------------------------------*/
   Show( true ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::build_menubar()
{
   /*-----------------------------------------------------------------------+
   ! Menu                                                                   !
   +-----------------------------------------------------------------------*/
   m_menubar = new wxMenuBar ;
   wxMenu *mp ;

   /*-----------------------------------------------------------------------+
   ! Menu File                                                              !
   +-----------------------------------------------------------------------*/
   mp = new wxMenu ;
   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, wxID_PREFERENCES, wxEmptyString,
                     preferences_desktop_16x16_xpm
                   ) ;
   /*----------------------------------------------------------------------*/
   mp->AppendSeparator() ;
   mtf::append_item( mp, wxID_EXIT, wxART_QUIT ) ;
   /*----------------------------------------------------------------------*/
   m_menubar->Append( mp, _( "&File" ) ) ;

   /*-----------------------------------------------------------------------+
   ! Menu Display                                                           !
   +-----------------------------------------------------------------------*/
   mp = new wxMenu ;
   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, ID_TOGGLE_MINI_DISPLAY, "_",
                     text_x_generic_template_16x16_xpm
                   ) ;
   mp->AppendSeparator() ;
   mp->AppendCheckItem( ID_STAY_ON_TOP, _( "Always on &Top" ),
                        _( "Keep this window on top of all the other ones" )
                      )->Check( wxGetApp().M_boo_stay_on_top.get() ) ;
   mp->AppendSeparator() ;
   mp->AppendCheckItem( ID_DISP_BOPL, _( "&Playlists" ) ) ;
   mp->AppendCheckItem( ID_DISP_PL, _( "&Channels" ) ) ;
   mp->AppendCheckItem( ID_DISP_CTRL, _( "&Play toolbar" ) ) ;
   /*----------------------------------------------------------------------*/
   m_menubar->Append( mp, _( "&Display" ) ) ;

   /*-----------------------------------------------------------------------+
   ! Menu Play                                                              !
   +-----------------------------------------------------------------------*/
   mp = new wxMenu ;
   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, ID_PLAY_START_PAUSE, "_",
                     media_playback_start_16x16_xpm
                   ) ;
   mtf::append_item( mp, ID_PLAY_STOP, _( "&Stop\tCtrl+S" ),
                     media_playback_stop_16x16_xpm, _( "Stop channel" )
                   ) ;
   mtf::append_item( mp, ID_PLAY_STOP_ALL,
                     _( "&Stop all channels\tCtrl+Shift+S" ),
                     media_playback_stop_16x16_xpm, _( "Stop all channels" )
                   ) ;

   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, ID_PLAY_PREV, _( "Play &previous\tCtrl+P" ),
                     media_skip_backward_16x16_xpm,
                     _( "Play previous channel in the current playlist" )
                   ) ;
   mtf::append_item( mp, ID_PLAY_NEXT, _( "Play &next\tCtrl+N" ),
                     media_skip_forward_16x16_xpm,
                     _( "Play next channel in the current playlist" )
                   ) ;
   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, ID_PLAY_HIST,
                     _( "Play channel played before\tCtrl+B" ),
                     image_loading_16x16_xpm,
                     _( "Play a recently played channel" )
                   ) ;
   m_menu_play_hist = new wxMenu ;
   mtf::append_item( mp, ID_PLAY_HIST_MENU, _( "&Recently played" ),
                     image_loading_16x16_xpm,
                     mp->FindItem( ID_PLAY_START_PAUSE )->GetHelp(),
                     wxITEM_NORMAL, m_menu_play_hist
                   ) ;
   refresh_menu_play_hist() ;
   mtf::append_item( mp, ID_FIND_PLAYED_ACTIVE,
                     _( "&Find played channel\tCtrl+G" ),
                     mtf::get_menu_bmp( wxART_FIND ),
                     _( "Find the active channel currently played" )
                   ) ;
   /*----------------------------------------------------------------------*/
   mp->AppendSeparator() ;
   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, ID_FULLSCREEN,
                     _( "&Change full screen mode\tCtrl+F" ),
                     mtf::get_menu_bmp( wxART_FULL_SCREEN ),
                     _( "Switch display to the next full screen mode" )
                   ) ;
   mtf::append_item( mp, ID_SNAPSHOT, _( "&Snapshot\tCtrl+Y" ),
                     camera_photo_16x16_xpm,
                     _( "Take a snapshot of the selected channel" )
                   ) ;
   /*----------------------------------------------------------------------*/
   mp->AppendSeparator() ;
   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, ID_VOLUME_UP, _( "&Increase volume\tCtrl+Up" ),
                     audio_volume_high_16x16_xpm,
                     _( "Increase volume" )
                   ) ;
   mtf::append_item( mp, ID_VOLUME_DOWN, _( "&Decrease volume\tCtrl+Down" ),
                     audio_volume_low_16x16_xpm,
                     _( "Decrease volume" )
                   ) ;
   mtf::append_item( mp, ID_TOGGLE_MUTE, "_",
                     audio_volume_muted_16x16_xpm,
                     _( "Enable or disable sound" )
                   ) ;
   /*----------------------------------------------------------------------*/
   m_menubar->Append( mp, _( "&Playback" ) ) ;
   m_menu_play = mp ;

   /*-----------------------------------------------------------------------+
   ! Menu ?                                                                 !
   +-----------------------------------------------------------------------*/
   mp = new wxMenu ;
   /*----------------------------------------------------------------------*/
   mtf::append_item( mp, ID_MAKE_DONATION, _( "Make a &donation" ),
                     system_users_16x16_xpm
                   ) ;
   mp->AppendSeparator() ;
   mtf::append_item( mp, ID_WEB_SITE_MAIN, _( "Web &site" ),
                     internet_web_browser_16x16_xpm
                   ) ;
   mtf::append_item( mp, ID_WEB_SITE_FORUM, _( "&Forum" ),
                     internet_web_browser_16x16_xpm
                   ) ;
   mtf::append_item( mp, ID_CHECK_VERSION_MANUAL, _( "&Check for update" ),
                     software_update_available_16x16_xpm
                   ) ;
   /*----------------------------------------------------------------------*/
   mp->AppendSeparator() ;
   mtf::append_item( mp, wxID_ABOUT, wxART_INFORMATION ) ;
   m_menubar->Append( mp, "&?" ) ;

   /*-----------------------------------------------------------------------+
   ! Menu options bindings                                                  !
   +-----------------------------------------------------------------------*/
   Bind( wxEVT_MENU, &CFrame::OnPrefs, this, wxID_PREFERENCES ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { Close() ; },
         wxID_EXIT
       ) ;
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  wxGetApp().M_boo_stay_on_top.toggle() ;
            ToggleWindowStyle( wxSTAY_ON_TOP ) ;
         },
         ID_STAY_ON_TOP
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  toggle_mini_display() ; },
         ID_TOGGLE_MINI_DISPLAY
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { toggle_pane_show( m_bopl ) ; },
         ID_DISP_BOPL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  toggle_pane_show( get_window_pl() ) ; },
         ID_DISP_PL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  toggle_pane_show( m_panel_ctrl ) ; },
         ID_DISP_CTRL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { play_start_pause() ; },
         ID_PLAY_START_PAUSE
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { close_media_active() ; },
         ID_PLAY_STOP
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { close_all_media() ; },
         ID_PLAY_STOP_ALL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  int i_num = event.GetId() - ID_PLAY_HIST_START ;
            play_channel( m_ds_play_hist[ i_num ] ) ;
         },
         ID_PLAY_HIST_START, ID_PLAY_HIST_END
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { play_prev_hil() ; },
         ID_PLAY_PREV
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { play_next_hil() ; },
         ID_PLAY_NEXT
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { play_hist() ; },
         ID_PLAY_HIST
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { play_hist_clear() ; },
         ID_PLAY_HIST_CLEAR
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  get_media_active()->show_played_channel_item() ; },
         ID_FIND_PLAYED_ACTIVE
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { next_fullscreen() ; },
         ID_FULLSCREEN
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { take_snapshot() ; },
         ID_SNAPSHOT
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_sld_volume->SetValue( m_sld_volume->GetValue() + 1 ) ;
            set_volume_from_slider() ;
         },
         ID_VOLUME_UP
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  m_sld_volume->SetValue( m_sld_volume->GetValue() - 1 ) ;
            set_volume_from_slider() ;
         },
         ID_VOLUME_DOWN
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event ) { toggle_mute() ; },
         ID_TOGGLE_MUTE
       ) ;
   Bind( wxEVT_MENU,
         []( wxCommandEvent &event )
         {  wxLaunchDefaultBrowser( MTF_URL_DONATION ) ; },
         ID_MAKE_DONATION
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  wxLaunchDefaultBrowser( MTF_URL_SITE ) ; },
         ID_WEB_SITE_MAIN
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  wxLaunchDefaultBrowser( MTF_URL_FORUM ) ; },
         ID_WEB_SITE_FORUM
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  /*--( Get version data )---------------------------------------*/
            CDownload dl( this, MTF_URL_VERSION ) ;
            int i_err = dl.non_threaded_exec() ;
            check_version( i_err, dl.get_def_data(), true ) ;
            /*-------------------------------------------------------------*/
         },
         ID_CHECK_VERSION_MANUAL
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  CDlgAbout( this ).get_modal() ; },
         wxID_ABOUT
       ) ;
   /*----------------------------------------------------------------------*/
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  toggle_show() ; },
         ID_TOGGLE_SHOW
       ) ;
   Bind( wxEVT_MENU,
         [ this ]( wxCommandEvent &event )
         {  toggle_caption() ; },
         ID_TOGGLE_CAPTION
       ) ;
   /*----------------------------------------------------------------------*/
}

/*--( Activated via idle event )-------------------------------------------*/
void CFrame::OnFirstIdle( wxIdleEvent &event )
{
   /*--( One execution is enough )-----------------------------------------*/
   Unbind( wxEVT_IDLE, &CFrame::OnFirstIdle, this ) ;
   /*-----------------------------------------------------------------------+
   ! "init_media_windows()" is called after the display is done because of  !
   ! the way "CVlcMedia::init_player_window" works under wxGTK.             !
   ! I have spent time on this one :                                        !
   ! - If during the "CVlcMedia" creation : "init_player_window" is called  !
   !   which calls "gtk_widget_realize"                                     !
   ! - In "wx/src/gtk/toplevel.cpp" : In "Show", "m_deferShow" is set to    !
   !   false, which in "GTKUpdateDecorSize" makes the decor size to be      !
   !   added                                                                !
   ! - The decor size is added to the frame size ... : w=4, h=26            !
   +-----------------------------------------------------------------------*/
   m_boo_first_idle_done = true ;
   /*-----------------------------------------------------------------------+
   ! This has to be done once everything else has been done, especially the !
   ! drawing of the frame. Else the visual rendering is not pleasant        !
   +-----------------------------------------------------------------------*/
   play_on_start() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnPrefs( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   CDlgPrefs().get_modal() ;
   /*--( Restart needed and chosen by the user ? )-------------------------*/
   if( !wxGetApp().get_program_to_start_on_exit().empty() )
   {  Close( true ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnBtPlayStop( wxAuiToolBarEvent &event )
{
   /*--( Dropdown list opened ? )------------------------------------------*/
   if( !event.IsDropDownClicked() )
   {  close_media_active() ; }
   else
   {  /*-------------------------------------------------------------------*/
      wxRect  rect = m_to_ctrl->GetToolRect( event.GetId() ) ;
      wxPoint pt   = m_to_ctrl->ClientToScreen( rect.GetBottomLeft() ) ;
      pt = ScreenToClient( pt ) ;
      /*-------------------------------------------------------------------*/
      wxMenu menu ;
      mtf::append_item( &menu, m_menubar->FindItem( ID_PLAY_STOP ) ) ;
      mtf::append_item( &menu, m_menubar->FindItem( ID_PLAY_STOP_ALL ) ) ;
      /*-------------------------------------------------------------------*/
      PopupMenu( &menu, pt ) ;
      /*--( Make sure the button is "un-stuck" )---------------------------*/
      m_to_ctrl->SetToolSticky( event.GetId(), false ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnBtPlayHist( wxAuiToolBarEvent &event )
{
   /*--( Dropdown list opened ? )------------------------------------------*/
   if( !event.IsDropDownClicked() )
   {  play_hist() ; }
   else
   {  /*-------------------------------------------------------------------*/
      wxRect  rect = m_to_ctrl->GetToolRect( event.GetId() ) ;
      wxPoint pt   = m_to_ctrl->ClientToScreen( rect.GetBottomLeft() ) ;
      pt = ScreenToClient( pt ) ;
      /*--( To be "freely" used the menu must be orphan )------------------*/
      wxMenu *menu_parent = m_menu_play_hist->GetParent() ;
      m_menu_play_hist->SetParent( nullptr ) ;
      PopupMenu( m_menu_play_hist, pt ) ;
      m_menu_play_hist->SetParent( menu_parent ) ;
      /*--( Make sure the button is "un-stuck" )---------------------------*/
      m_to_ctrl->SetToolSticky( event.GetId(), false ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnFullScreenMouseOn( wxMouseEvent &event )
{
   /*----------------------------------------------------------------------*/
   get_media_active()->SetCursor( wxCursor( wxNullCursor ) ) ;
   m_timer_mouse_off.StartOnce( 3000 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnFullScreenMouseOff( wxTimerEvent &event )
{
   /*--( When the popup menu is opened the cursor has to be kept visible )-*/
   if( !get_media_active()->get_popup_menu_on() )
   {  get_media_active()->SetCursor( wxCursor( wxCURSOR_BLANK ) ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnRefreshViewers( wxTimerEvent &event )
{
   /*--( At least one media is played ? )----------------------------------*/
   CVlcMedia *p_media_active = get_media_active() ;
   if( p_media_active == nullptr || !p_media_active->is_playing() )
   {  return ; }
   /*--( The number of viewers for this channel can be retrieved ? )-------*/
   CChannel *p_channel = p_media_active->get_channel_played() ;
   if( p_channel == nullptr || !p_channel->has_nb_viewers() ) { return ; }
   /*--( The number of viewers is displayed in the title )-----------------*/
   set_title() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::OnSldVolumeMouseWheel( wxMouseEvent &event )
{
   /*--( Use VLC wheel move value increment )------------------------------*/
   int i_value =   m_sld_volume->GetValue()
                 + ( event.GetWheelRotation() > 0 ? +1 : -1 ) ;
   /*--( Keep the new value in the new interval )--------------------------*/
   if( i_value > m_sld_volume->GetMax() )
   {  i_value = m_sld_volume->GetMax() ; }
   else
   if( i_value < m_sld_volume->GetMin() )
   {  i_value = m_sld_volume->GetMin() ; }
   /*--( Set it ... )------------------------------------------------------*/
   m_sld_volume->SetValue( i_value ) ;
   set_volume_from_slider() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::set_title()
{
   /*----------------------------------------------------------------------*/
   wxString s_title ;
   CVlcMedia *p_media_active = get_media_active() ;
   CChannel  *p_channel ;
   bool boo_refresh_viewers = false ;

   /*----------------------------------------------------------------------*/
   if(    p_media_active == nullptr
       || ( p_channel = p_media_active->get_channel_played() ) == nullptr
     )
   {  s_title = MTF_DISPLAY_NAME + " " MTF_VERSION ; }
   else
   {  /*-------------------------------------------------------------------*/
      s_title = p_channel->get_title() ;
      /*--( The number of viewers of the channel may be known )------------*/
      if( p_channel->has_nb_viewers() && p_media_active->is_playing() )
      {  /*----------------------------------------------------------------*/
         if( p_channel->is_nb_viewers_refresh_needed() )
         {  /*-------------------------------------------------------------*/
            refresh_nb_viewers( p_channel ) ;
            /*--------------------------------------------------------------+
            ! No title display this time, it will be done after the data    !
            ! have been fetched and this function called again              !
            +--------------------------------------------------------------*/
            return ;
            /*-------------------------------------------------------------*/
         }
         /*--( We will refresh the number of viewers later )---------------*/
         boo_refresh_viewers = true ;
         /*----------------------------------------------------------------*/
         long l_nb = p_channel->get_nb_viewers() ;
         if( l_nb > 0 )
         {  s_title +=   " - "
                       + wxString::Format( wxPLURAL( "%ld viewer",
                                                     "%ld viewers", l_nb
                                                   ), l_nb
                                         ) ;
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }

   /*-----------------------------------------------------------------------+
   ! Do we repeat or stop the viewers number refresh ?                      !
   ! If nothing is played or if the channel doesn't provide the number of   !
   ! viewers there is no need to do it : the timer is stopped               !
   ! Else it is restarted to refresh the data later                         !
   +-----------------------------------------------------------------------*/
   if( boo_refresh_viewers != m_timer_viewers.IsRunning() )
   {  start_viewer_timer( boo_refresh_viewers ) ; }

   /*--( Stop here if nothing changed )------------------------------------*/
   if( s_title == GetTitle() ) { return ; }
   /*----------------------------------------------------------------------*/
   SetTitle( s_title ) ;
   /*-----------------------------------------------------------------------+
   ! Set icon tooltip to the title. But there is no function to change it   !
   ! only so the icon is set too                                            !
   +-----------------------------------------------------------------------*/
   if( m_taskbar_icon != nullptr )
   {  m_taskbar_icon->SetIcon( GetIcon(), s_title ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::save_display_state()
{
   /*----------------------------------------------------------------------*/
   if( wxGetApp().M_boo_mini_display.get() )
   {  mtf::save_tlw_rect( this, wxGetApp().M_main_mini_rec ) ; }
   else
   {  /*-------------------------------------------------------------------*/
      mtf::save_tlw_rect( this, wxGetApp().M_main_rec ) ;
      wxGetApp().M_s_main_perspective.set( m_auimgr.SavePerspective() ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::setup_display_rect()
{
   /*----------------------------------------------------------------------*/
   if( wxGetApp().M_boo_mini_display.get() )
   {  mtf::tlw_set_rect( this, wxGetApp().M_main_mini_rec.get() ) ; }
   else
   {  mtf::tlw_set_rect( this, wxGetApp().M_main_rec.get() ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::setup_display_mode()
{
   /*----------------------------------------------------------------------*/
   long l_style = GetWindowStyleFlag() ;
   const bool co_boo_normal = !wxGetApp().M_boo_mini_display.get() ;
   /*--( If the menubar is not displayed the shortcuts won't be active )---*/
   if( co_boo_normal )
   {
      /*-------------------------------------------------------------------*/
      if( wxGetApp().M_boo_stay_on_top.get() )
      {  l_style |= wxSTAY_ON_TOP ; }
      else
      {  l_style &= ~wxSTAY_ON_TOP ; }
      /*-------------------------------------------------------------------*/
      if( !wxGetApp().M_s_main_perspective.empty() )
      {  m_auimgr.LoadPerspective( wxGetApp().M_s_main_perspective.get() ) ;}
      /*-------------------------------------------------------------------*/
      if( m_boo_mini_display_smallest )
      {  /*--( In normal display the caption is always shown )-------------*/
         l_style |= wxCAPTION ;
         /*----------------------------------------------------------------*/
         SetMenuBar( m_menubar ) ;
         SetAcceleratorTable( *m_menubar->GetAcceleratorTable() ) ;
         /*----------------------------------------------------------------*/
      }
      /*--( Restore window and menu )--------------------------------------*/
      Show() ;
      /*-------------------------------------------------------------------*/
   }
   else
   {
      /*--( The mini display is alway on top )-----------------------------*/
      l_style |= wxSTAY_ON_TOP ;
      /*-------------------------------------------------------------------*/
      m_auimgr.GetPane( m_bopl ).Show( false ) ;
      m_auimgr.GetPane( get_window_pl() ).Show( false ) ;
      m_auimgr.GetPane( m_panel_ctrl ).Show( false ) ;
      /*--( Remove menu but keep accelerators )----------------------------*/
      if( m_boo_mini_display_smallest )
      {  /*----------------------------------------------------------------*/
         if( wxGetApp().M_boo_mini_display_caption.get() )
         {  l_style |= wxCAPTION ; }
         else
         {  l_style &= ~wxCAPTION ; }
         /*-----------------------------------------------------------------+
         ! If the menubar is not displayed the shortcuts are not active.    !
         ! Therefore they are set separately. Note that the "GetAccel..."   !
         ! function returns the expected result only on MSW.                !
         +-----------------------------------------------------------------*/
         SetMenuBar( nullptr ) ;
         SetAcceleratorTable( *m_menubar->GetAcceleratorTable() ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*--( Init "Display" menu )---------------------------------------------*/
   init_gui_pane() ;
   m_menubar->Check( ID_STAY_ON_TOP, ( l_style & wxSTAY_ON_TOP ) != 0 ) ;
   m_menubar->Enable( ID_STAY_ON_TOP, co_boo_normal ) ;
   m_menubar->Enable( ID_DISP_BOPL,   co_boo_normal ) ;
   m_menubar->Enable( ID_DISP_PL,     co_boo_normal ) ;
   m_menubar->Enable( ID_DISP_CTRL,   co_boo_normal ) ;
   /*--( No fullscreen in mini display mode )------------------------------*/
   m_menubar->Enable( ID_FULLSCREEN, co_boo_normal ) ;
   /*----------------------------------------------------------------------*/
   SetWindowStyle( l_style ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::play_on_start()
{
   /*--( Automatically play last channels played ? )-----------------------*/
   if(    !wxGetApp().M_boo_startup_play.get()
       || wxGetApp().M_s_startup_play_list.empty()
     )
   {  return ; }

   /*--( The previously channels list is in a string )---------------------*/
   wxStringTokenizer tkz( wxGetApp().M_s_startup_play_list.get(), '\n' ) ;
   mtf::t_vec_string vs_channel ;
   /*----------------------------------------------------------------------*/
   while( tkz.HasMoreTokens() )
   {  /*-------------------------------------------------------------------*/
      wxString s_channel = tkz.GetNextToken() ;
      /*--------------------------------------------------------------------+
      ! Check if channel is available. This cannot be the case for example  !
      ! when a live channel is played then deleted (the channel is always   !
      ! playing) and then the program quitted.                              !
      ! And an unknown channel will during next startup ...                 !
      +--------------------------------------------------------------------*/
      if( wxGetApp().get_channel( s_channel ) )
      {  vs_channel.emplace_back( s_channel ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*--( The active channel previously played has to be set too )----------*/
   play_channel( vs_channel, wxGetApp().M_s_startup_play_active.get() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CPlaylistUser *CFrame::create_new_pl_user()
{
   /*----------------------------------------------------------------------*/
   wxString s_name ;
   /*----------------------------------------------------------------------*/
   CDlgPlName( _( "New Playlist" ) ).get_modal( s_name ) ;
   /*--( If the returned name is empty, this means an error occurred )-----*/
   if( s_name.empty() ) { return( nullptr ) ; }
   /*----------------------------------------------------------------------*/
   CPlaylistUser *pl = new CPlaylistUser( this, s_name, s_name,
                                          wxGetApp().m_s_tree_pl_user_path
                                        ) ;
   m_bopl->add_pl( pl ) ;
   m_bopl->EnsureVisible( pl->get_ti() ) ;
   /*----------------------------------------------------------------------*/
   return( pl ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CVlcMedia *CFrame::new_media( const wxString &s_for_channel )
{
   /*--( Display changes are useless to be showed )------------------------*/
   wxWindowUpdateLocker no_window_updates( m_panel_media ) ;
   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media = new CVlcMedia( m_panel_media, s_for_channel ) ;
   if( !p_media->is_ok() )
   {  delete_media( p_media ) ;
      return( nullptr ) ;
   }
   /*----------------------------------------------------------------------*/
   m_vec_media.emplace_back( p_media ) ;
   /*----------------------------------------------------------------------*/
   if( m_vec_media.size() == 1 || wxGetApp().M_boo_new_media_active.get() )
   {  set_media_active( p_media ) ; }
   /*----------------------------------------------------------------------*/
   layout_media() ;
   /*----------------------------------------------------------------------*/
   return( p_media ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::delete_media( CVlcMedia *p_media )
{
   /*----------------------------------------------------------------------*/
   p_media->refresh_pl_channel() ;
   delete p_media ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CFrame::reinit_current_media( const wxString &s_for_channel )
{
   /*----------------------------------------------------------------------*/
   int i_media = find_media( get_media_active() ) ;
   if( i_media < 0 ) { return( -1 ) ; }

   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media = new CVlcMedia( m_panel_media, s_for_channel ) ;
   if( !p_media->is_ok() )
   {  delete_media( p_media ) ;
      return( -2 ) ;
   }
   /*----------------------------------------------------------------------*/
   m_bsz_media->Replace( get_media_active()->get_window(),
                         p_media->get_window(),
                         true
                       ) ;
   /*----------------------------------------------------------------------*/
   delete_media( get_media_active() ) ;
   m_vec_media[ i_media ] = p_media ;
   /*----------------------------------------------------------------------*/
   m_bsz_media->Layout() ;
   set_media_active( p_media ) ;
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*--( The layout is based on the most near square )------------------------*/
void CFrame::layout_media()
{
   /*--( First : cleaning. The media are not deleted )---------------------*/
   while( m_bsz_media->GetItemCount() != 0 )
   {  /*-------------------------------------------------------------------*/
      wxSizer *bsz = m_bsz_media->GetItem( static_cast< size_t >( 0 )
                                         )->GetSizer() ;
      bsz->Clear() ;
      m_bsz_media->Detach( 0 ) ;
      delete bsz ;
      /*-------------------------------------------------------------------*/
   }
   /*--( Size of rows/cols of the layout square )--------------------------*/
   int co_i_nb_media = static_cast< int >( m_vec_media.size() ) ;
   int i_nb_col ;
   int i_nb_row ;
   /*--( Some cases have to be manually "changed" )------------------------*/
   if( co_i_nb_media == 2 )
   {  i_nb_row = 2 ; i_nb_col = 1 ; }
   else
   {  i_nb_col = std::ceil( std::sqrt( co_i_nb_media ) ) ;
      i_nb_row = std::ceil(   static_cast< double >( co_i_nb_media )
                            / i_nb_col
                          ) ;
   }

   /*--( Then recreate one row after another )-----------------------------*/
   int i_media = 0 ;
   for( int i_row = 0 ; i_row < i_nb_row ; ++i_row )
   {
      /*-------------------------------------------------------------------*/
      wxBoxSizer *bsz_row = new wxBoxSizer( wxHORIZONTAL ) ;
      m_bsz_media->Add( bsz_row, 1, wxEXPAND ) ;
      /*--( And each cell one after another )------------------------------*/
      for( int i_col = 0 ; i_col < i_nb_col && i_media < co_i_nb_media ;
           ++i_col, ++i_media
         )
      {  /*----------------------------------------------------------------*/
         wxSizerItem *szi
         = bsz_row->Add( m_vec_media[ i_media ]->get_window(), 1, wxEXPAND );
         /*--( Media hidden ? )--------------------------------------------*/
         szi->Show(    m_fullscreen_level != FSL_ONE_MEDIA
                    || get_media_active() == m_vec_media[ i_media ]
                  ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   m_bsz_media->Layout() ;
   /*--( From one fullscreen to another the focus is set or not )----------*/
   if(    get_media_active() != nullptr
       && m_fullscreen_level != FSL_ALL_MEDIA
       && m_vec_media.size() > 1
     )
   {  get_media_active()->show_focus() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::set_media_active( CVlcMedia *p_media )
{
   /*--( Useless ? )-------------------------------------------------------*/
   if( m_p_media_active == p_media ) { return ; }
   /*--( Set new one (even if nullptr) )-----------------------------------*/
   CVlcMedia *p_media_previous = m_p_media_active ;
   m_p_media_active = p_media ;
   /*--( Reset previous if always present )--------------------------------*/
   if( p_media_previous != nullptr && find_media( p_media_previous ) >= 0 )
   {
      /*-------------------------------------------------------------------*/
      p_media_previous->show_focus() ;
      /*-------------------------------------------------------------------*/
      if( p_media_previous->is_playing() )
      {  p_media_previous->init_audio() ; }
      /*--( Font used for the channel name changes if it is active or not )*/
      p_media_previous->refresh_pl_channel() ;
      /*-------------------------------------------------------------------*/
   }
   /*--( Set new )---------------------------------------------------------*/
   if( m_p_media_active != nullptr )
   {
      /*-------------------------------------------------------------------*/
      m_p_media_active->show_focus() ;
      /*-------------------------------------------------------------------*/
      if( m_p_media_active->is_playing() )
      {  m_p_media_active->init_audio() ; }
      /*--( Font used for the channel name changes if it is active or not )*/
      m_p_media_active->refresh_pl_channel() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   set_title() ;
   init_gui_play() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
bool CFrame::set_media_active( size_t sz_num )
{
   /*----------------------------------------------------------------------*/
   if( sz_num < 0 || sz_num >= m_vec_media.size() )
   {  return( false ) ; }
   /*----------------------------------------------------------------------*/
   set_media_active( m_vec_media[ sz_num ] ) ;
   return( true ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
bool CFrame::set_media_active( const wxString &s_channel )
{
   /*----------------------------------------------------------------------*/
   if( !s_channel.empty() )
   {   /*------------------------------------------------------------------*/
       for( auto p_media : m_vec_media )
       {  /*---------------------------------------------------------------*/
          if( p_media->get_channel_name() == s_channel )
          {  set_media_active( p_media ) ;
             return( true ) ;
          }
          /*---------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( false ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CFrame::find_media( const CVlcMedia *p_media )
{
   /*----------------------------------------------------------------------*/
   auto cit = std::find( m_vec_media.begin(), m_vec_media.end(), p_media ) ;
   return( cit == m_vec_media.end() ? -1 : cit - m_vec_media.begin() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CVlcMedia *CFrame::find_media( const CChannel *p_channel )
{
   /*----------------------------------------------------------------------*/
   auto cit = std::find_if( m_vec_media.begin(), m_vec_media.end(),
                            [ p_channel ]( const CVlcMedia *p_media )
                            {  return(    p_channel
                                       == p_media->get_channel_played()
                                     ) ;
                            }
                          ) ;
   return( cit == m_vec_media.end() ? nullptr : *cit ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CVlcMedia *CFrame::find_media( const wxString &s_channel )
{
   /*----------------------------------------------------------------------*/
   CChannel *p_channel = wxGetApp().get_channel( s_channel ) ;
   /*----------------------------------------------------------------------*/
   auto cit = std::find_if( m_vec_media.begin(), m_vec_media.end(),
                            [ p_channel ]( const CVlcMedia *p_media )
                            {  return(    p_channel
                                       == p_media->get_channel_played()
                                     ) ;
                            }
                          ) ;
   return( cit == m_vec_media.end() ? nullptr : *cit ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
bool CFrame::check_nb_media( int i_tot_nb )
{
   /*----------------------------------------------------------------------*/
   const int co_i_nb_max = wxGetApp().M_i_max_nb_media.get() ;
   /*--( The question is asked only once when the limit is crossed )-------*/
   if(    co_i_nb_max != 0
       && static_cast< int >( m_vec_media.size() ) <= co_i_nb_max
       && i_tot_nb > co_i_nb_max
     )
   {  /*-------------------------------------------------------------------*/
      wxMessageDialog dlg( this,
                           wxString::Format( _( "Are you sure you want to "
                                                "play more than %d channels "
                                                "simultaneously ?"
                                              ), co_i_nb_max
                                           ),
                           _( "Really ?" ),
                             wxICON_WARNING | wxYES_NO | wxCANCEL
                           | wxCANCEL_DEFAULT
                         ) ;
      /*--------------------------------------------------------------------+
      ! A bit "tricky" here. The action associated to buttons are :         !
      ! Yes    : Let's GO                                                   !
      ! No     : Yes and "don't ask again"                                  !
      ! Cancel : No                                                         !
      +--------------------------------------------------------------------*/
      dlg.SetYesNoCancelLabels( wxID_YES, _( "&No need to ask" ), wxID_NO ) ;
      dlg.SetExtendedMessage( _( "Without enough cpu power and bandwidth "
                                 "this program can become unstable"
                               )
                            ) ;
      /*-------------------------------------------------------------------*/
      int i_choice = dlg.ShowModal() ;
      /*-------------------------------------------------------------------*/
      if( i_choice == wxID_CANCEL )
      {  return( false ) ; }
      /*--( "Don't ask again" => 0 means no limit )------------------------*/
      if( i_choice == wxID_NO )
      {  wxGetApp().M_i_max_nb_media.set( 0 ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( true ) ;
   /*----------------------------------------------------------------------*/
}

/*--( By default the channel is played in active media )-------------------*/
void CFrame::play_channel( const wxString &s_channel, bool boo_new_media )
{
   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media ;
   /*--( Play in a new media ? )-------------------------------------------*/
   if( !boo_new_media && !m_vec_media.empty() )
   {  /*-------------------------------------------------------------------*/
      if( reinit_current_media( s_channel ) != 0 ) { return ; }
      /*-------------------------------------------------------------------*/
      p_media = get_media_active() ;
      if( p_media == nullptr ) { return ; }
      /*-------------------------------------------------------------------*/
   }
   else
   {  /*-------------------------------------------------------------------*/
      if( !check_nb_media( m_vec_media.size() + 1 ) ) { return ; }
      /*-------------------------------------------------------------------*/
      p_media = new_media( s_channel ) ;
      if( p_media == nullptr ) { return ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   p_media->start_channel() ;
   /*----------------------------------------------------------------------*/
}

/*--( Channels are played in new media )-----------------------------------*/
void CFrame::play_channel( const mtf::t_vec_string &vs_channel,
                           const wxString &s_active
                         )
{
   /*----------------------------------------------------------------------*/
   const size_t co_sz_nb_channel = vs_channel.size() ;
   /*----------------------------------------------------------------------*/
   if( co_sz_nb_channel == 0 ) { return ; }

   /*-----------------------------------------------------------------------+
   ! We assume that we know what we do if the active channel is set. This   !
   ! is the case for "play on start"                                        !
   +-----------------------------------------------------------------------*/
   if( s_active.empty() && !check_nb_media( co_sz_nb_channel ) )
   {  return ; }

   /*--( Remove current media )--------------------------------------------*/
   close_all_media() ;
   /*--( Reallocate the ones needed )--------------------------------------*/
   {
      /*-------------------------------------------------------------------*/
      wxWindowUpdateLocker no_window_updates( m_panel_media ) ;
      /*-------------------------------------------------------------------*/
      for( size_t sz_num = 0 ; sz_num < co_sz_nb_channel ; ++sz_num )
      {  /*--( Ko can be because of invalid vlc path )---------------------*/
         auto *p_media = new CVlcMedia( m_panel_media, vs_channel[ sz_num ]);
         if( !p_media->is_ok() )
         {  close_all_media() ;
            return ;
         }
         /*----------------------------------------------------------------*/
         m_vec_media.emplace_back( p_media ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      layout_media() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   for( auto p_media : m_vec_media ) { p_media->show_focus() ; }

   /*--( The action is split into two parts for a better looking result )--*/
   for( size_t sz_num = 0 ; sz_num < co_sz_nb_channel ; ++sz_num )
   {  m_vec_media[ sz_num ]->start_channel() ; }

   /*--( By default the active media is the first one )--------------------*/
   if( !set_media_active( s_active ) )
   {  set_media_active( m_vec_media[ 0 ] ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CFrame::close_media( CVlcMedia *p_media, bool boo_layout )
{
   /*--( Display changes showed later )------------------------------------*/
   wxWindowUpdateLocker no_window_updates( m_panel_media ) ;
   /*----------------------------------------------------------------------*/
   if( p_media == nullptr ) { return( -1 ) ; }
   /*----------------------------------------------------------------------*/
   int i_media = find_media( p_media ) ;
   if( i_media < 0 ) { return( -2 ) ; }
   /*--( Back from normal display when the last media is fullscreen )------*/
   if( nb_media_to_show() == 1 && get_fullscreen_level() != FSL_NONE )
   {  set_fullscreen( FSL_NONE ) ; }
   /*--( Take the window out the sizer )-----------------------------------*/
   wxSizer *sizer = p_media->get_window()->GetSizer() ;
   if( sizer == nullptr )
   {  return( -3 ) ; }
   sizer->Detach( p_media->get_window() ) ;
   /*--( Remove data associated to this media )----------------------------*/
   delete_media( m_vec_media[ i_media ] ) ;
   m_vec_media.erase( m_vec_media.begin() + i_media ) ;
   /*--( Choose the new "active" window )----------------------------------*/
   if( p_media == m_p_media_active )
   {  /*-------------------------------------------------------------------*/
      if( m_vec_media.empty() )
      {  set_media_active( nullptr ) ; }
      else
      {  /*----------------------------------------------------------------*/
         if( i_media >= static_cast< int >( m_vec_media.size() ) )
         {  set_media_active( m_vec_media.back() ) ; }
         else
         {  set_media_active( m_vec_media[ i_media ] ) ; }
         /*--( Another media gets the focus )------------------------------*/
         m_p_media_active->SetFocus() ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*--( If only one is remaining it doesn't need to be framed )-----------*/
   if( m_vec_media.size() == 1 ) { m_p_media_active->show_focus() ; }
   /*----------------------------------------------------------------------*/
   if( boo_layout ) { layout_media() ; }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::close_all_media()
{
   /*----------------------------------------------------------------------*/
   if( m_vec_media.empty() ) { return ; }
   /*--( Display changes showed later )------------------------------------*/
   wxWindowUpdateLocker no_window_updates( m_panel_media ) ;
   /*----------------------------------------------------------------------*/
   for( auto p_media : m_vec_media ) { delete_media( p_media ) ; }
   /*----------------------------------------------------------------------*/
   m_vec_media.clear() ;
   m_p_media_active = nullptr ;
   /*----------------------------------------------------------------------*/
   layout_media() ;
   set_title() ;
   init_gui_play() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::refresh_menu_play_hist()
{
   /*--( Clear menu and not recreate it to keep the menubar tree ok )------*/
   while( m_menu_play_hist->GetMenuItemCount() > 0 )
   {  m_menu_play_hist->Delete( m_menu_play_hist->FindItemByPosition( 0 ) );}
   /*--( Recreate the play history menu items )----------------------------*/
   int i_id = ID_PLAY_HIST_START ;
   for( const auto &hist : m_ds_play_hist )
   {
      /*-------------------------------------------------------------------*/
      m_menu_play_hist->Append( i_id++, hist ) ;
      if( i_id >= ID_PLAY_HIST_END ) { break ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   m_menu_play_hist->AppendSeparator() ;
   m_menu_play_hist->Append( ID_PLAY_HIST_CLEAR, _( "Clear list" ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::toggle_mute()
{
   /*----------------------------------------------------------------------*/
   wxGetApp().M_boo_mute.toggle() ;
   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media_active = get_media_active() ;
   if( p_media_active != nullptr )
   {  p_media_active->init_audio() ; }
   /*----------------------------------------------------------------------*/
   init_gui_mute() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::set_volume_from_slider()
{
   /*----------------------------------------------------------------------*/
   wxGetApp().M_i_volume.set( m_sld_volume->GetValue() ) ;
   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media_active = get_media_active() ;
   if( p_media_active != nullptr ) { p_media_active->init_audio() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::toggle_pane_show( wxWindow *win_pane )
{
   /*--( No pane display change in "mini display" )------------------------*/
   if( wxGetApp().M_boo_mini_display.get() ) { return ; }
   /*----------------------------------------------------------------------*/
   wxAuiPaneInfo &pane = m_auimgr.GetPane( win_pane ) ;
   /*----------------------------------------------------------------------*/
   pane.Show( !pane.IsShown() ) ;
   init_gui_pane() ;
   /*----------------------------------------------------------------------*/
}

/*--( It looks more "right" if the floating size is the current size )-----*/
void CFrame::init_pane_floating_size( wxWindow *win_pane )
{
   /*----------------------------------------------------------------------*/
   wxAuiPaneInfo &pane = m_auimgr.GetPane( win_pane ) ;
   /*----------------------------------------------------------------------*/
   if( pane.floating_size == wxDefaultSize )
   {  pane.floating_size = win_pane->GetSize() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::start_viewer_timer( bool boo_start )
{
   /*----------------------------------------------------------------------*/
   if( boo_start )
   {  m_timer_viewers.Start( wxGetApp().M_i_viewers_delay.get() * 1000 ) ; }
   else
   {  m_timer_viewers.Stop() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::restart_viewer_timer_if_started()
{
   /*----------------------------------------------------------------------*/
   if( m_timer_viewers.IsRunning() ) { start_viewer_timer( true ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_gui_pane()
{
   /*----------------------------------------------------------------------*/
   m_menubar->Check( ID_DISP_BOPL,
                     m_auimgr.GetPane( m_bopl ).IsShown()
                   ) ;
   m_menubar->Check( ID_DISP_PL,
                     m_auimgr.GetPane( get_window_pl() ).IsShown()
                   ) ;
   m_menubar->Check( ID_DISP_CTRL,
                     m_auimgr.GetPane( m_panel_ctrl ).IsShown()
                   ) ;
   /*----------------------------------------------------------------------*/
   m_auimgr.Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_gui_play_hist()
{
   /*----------------------------------------------------------------------*/
   bool boo_play_hist_prev = ( !m_ds_play_hist.empty() ) ;
   m_menubar->Enable( ID_PLAY_HIST, boo_play_hist_prev ) ;
   m_to_ctrl->EnableTool( ID_PLAY_HIST, boo_play_hist_prev ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_gui_play()
{
   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media_active = get_media_active() ;
   /*----------------------------------------------------------------------*/
   bool boo_media_active_ok = ( p_media_active != nullptr ) ;
   libvlc_state_t state_media ;
   bool boo_retry ;
   /*----------------------------------------------------------------------*/
   if( boo_media_active_ok )
   {  state_media = p_media_active->get_state() ;
      boo_retry = p_media_active->get_play_retry() ;
   }
   else
   {  state_media = libvlc_NothingSpecial ;
      boo_retry = false ;
   }
   /*----------------------------------------------------------------------*/
   bool boo_playing = ( state_media == libvlc_Playing || boo_retry ) ;
   bool boo_paused = ( state_media == libvlc_Paused ) ;
   bool boo_active = ( boo_playing || boo_paused ) ;
   wxMenuItem *mi = m_menubar->FindItem( ID_PLAY_START_PAUSE ) ;
   /*----------------------------------------------------------------------*/
   wxString s_label ;
   wxString s_help ;
   if( boo_playing )
   {  s_label = _( "&Pause\tCtrl+Space" ) ;
      s_help = _( "Pause main channel" ) ;
   }
   else
   if( boo_paused )
   {  s_label = _( "&Resume\tCtrl+Space" ) ;
      s_help = _( "Resume main channel" ) ;
   }
   else
   {  s_label = _( "&Play\tCtrl+Space" ) ;
      s_help = _( "Play main channel" ) ;
   }
   /*----------------------------------------------------------------------*/
   mi->SetItemLabel( s_label ) ;
   mi->SetHelp( s_help ) ;
   mi->SetBitmap( boo_playing
                  ? media_playback_pause_16x16_xpm
                  : media_playback_start_16x16_xpm
                ) ;
   m_to_ctrl->SetToolBitmap( ID_PLAY_START_PAUSE,
                             boo_playing
                             ? media_playback_pause_32x32_xpm
                             : media_playback_start_32x32_xpm
                           ) ;
   m_to_ctrl->SetToolShortHelp( ID_PLAY_START_PAUSE, s_help ) ;
   /*----------------------------------------------------------------------*/
   m_menubar->Enable( ID_PLAY_START_PAUSE, !boo_retry ) ;
   m_to_ctrl->EnableTool( ID_PLAY_START_PAUSE, !boo_retry ) ;
   /*----------------------------------------------------------------------*/
   m_menubar->Enable( ID_PLAY_STOP, boo_media_active_ok ) ;
   m_to_ctrl->EnableTool( ID_PLAY_STOP, boo_media_active_ok ) ;
   m_menubar->Enable( ID_PLAY_STOP_ALL, boo_media_active_ok ) ;
   m_to_ctrl->EnableTool( ID_PLAY_STOP_ALL, boo_media_active_ok ) ;
   /*----------------------------------------------------------------------*/
   m_menubar->Enable( ID_FIND_PLAYED_ACTIVE, boo_active ) ;
   /*----------------------------------------------------------------------*/
   bool boo_on = boo_active && !boo_retry ;
   m_menubar->Enable( ID_SNAPSHOT, boo_on ) ;
   m_to_ctrl->EnableTool( ID_SNAPSHOT, boo_on ) ;
   /*----------------------------------------------------------------------*/
   init_gui_play_hist() ;
   /*----------------------------------------------------------------------*/
   m_auimgr.Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_gui_mini_display()
{
   /*----------------------------------------------------------------------*/
   bool boo_on = wxGetApp().M_boo_mini_display.get() ;
   wxMenuItem *mi = m_menubar->FindItem( ID_TOGGLE_MINI_DISPLAY ) ;
   /*----------------------------------------------------------------------*/
   mi->SetItemLabel( boo_on ? _( "&Normal display\tCtrl+H" )
                            : _( "&Minimal display\tCtrl+H" )
                   ) ;
   wxString s_help = ( boo_on ? _( "Switch to normal display" )
                              : _( "Switch to minimalistic display" )
                     ) ;
   mi->SetHelp( s_help ) ;
   m_to_ctrl->ToggleTool( ID_TOGGLE_MINI_DISPLAY, boo_on ) ;
   m_to_ctrl->SetToolShortHelp( ID_TOGGLE_MINI_DISPLAY, s_help ) ;
   /*----------------------------------------------------------------------*/
   m_auimgr.Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_gui_fullscreen()
{
   /*----------------------------------------------------------------------*/
   bool boo_on = ( get_fullscreen_level() != FSL_NONE ) ;
   /*----------------------------------------------------------------------*/
   m_to_ctrl->ToggleTool( ID_FULLSCREEN, boo_on ) ;
   /*----------------------------------------------------------------------*/
   m_auimgr.Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::init_gui_mute()
{
   /*----------------------------------------------------------------------*/
   bool boo_mute = wxGetApp().M_boo_mute.get() ;
   /*----------------------------------------------------------------------*/
   wxMenuItem *mi = m_menubar->FindItem( ID_TOGGLE_MUTE ) ;
   mi->SetItemLabel( boo_mute ? _( "&Unmute\tCtrl+M" )
                              : _( "&Mute\tCtrl+M" )
                   ) ;
   mi->SetBitmap( boo_mute ? audio_volume_medium_16x16_xpm
                           : audio_volume_muted_16x16_xpm
                ) ;
   /*----------------------------------------------------------------------*/
   m_to_ctrl->SetToolBitmap( ID_TOGGLE_MUTE,
                             boo_mute
                             ? audio_volume_muted_32x32_xpm
                             : audio_volume_medium_32x32_xpm
                           ) ;
   m_to_ctrl->ToggleTool( ID_TOGGLE_MUTE, boo_mute ) ;
   /*----------------------------------------------------------------------*/
   m_auimgr.Update() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::play_start_pause()
{
   /*----------------------------------------------------------------------*/
   libvlc_state_t state_media = ( m_p_media_active == nullptr
                                  ? libvlc_NothingSpecial
                                  : m_p_media_active->get_state()
                                ) ;
   /*----------------------------------------------------------------------*/
   if( state_media != libvlc_Playing && state_media != libvlc_Paused )
   {  play_focus() ; }
   else
   {  m_p_media_active->toggle_pause() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::play_focus()
{
   /*----------------------------------------------------------------------*/
   CPlaylist *pl = get_playlist_active() ;
   if( pl != nullptr ) { pl->play_focus() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::play_prev_hil()
{
   /*----------------------------------------------------------------------*/
   CPlaylist *pl = get_playlist_active() ;
   if( pl != nullptr ) { pl->play_prev_hil() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::play_next_hil()
{
   /*----------------------------------------------------------------------*/
   CPlaylist *pl = get_playlist_active() ;
   if( pl != nullptr ) { pl->play_next_hil() ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::play_hist()
{
   /*----------------------------------------------------------------------*/
   if( m_p_media_active == nullptr ) { return ; }
   /*--( Play last one if nothing currently played else play previous one )*/
   if( !m_p_media_active->is_running() && !m_ds_play_hist.empty() )
   {  play_channel( m_ds_play_hist[ 0 ] ) ; }
   else
   if( m_ds_play_hist.size() >= 2 )
   {  play_channel( m_ds_play_hist[ 1 ] ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::play_hist_clear()
{
   /*----------------------------------------------------------------------*/
   m_ds_play_hist.clear() ;
   refresh_menu_play_hist() ;
   init_gui_play_hist() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::toggle_mini_display()
{
   /*--( Save current state )----------------------------------------------*/
   save_display_state() ;
   /*--( Toggle value and force check because of "automatic" call )--------*/
   wxGetApp().M_boo_mini_display.toggle() ;
   /*--( Restore based on the new value of "boo mini display" )------------*/
   setup_display_rect() ;
   setup_display_mode() ;
   /*----------------------------------------------------------------------*/
   init_gui_mini_display() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::toggle_show()
{
   /*----------------------------------------------------------------------*/
   Show( !IsShown() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::toggle_caption()
{
   /*----------------------------------------------------------------------*/
   if(    !wxGetApp().M_boo_mini_display.get()
       || !m_boo_mini_display_smallest
     )
   {  return ; }
   /*----------------------------------------------------------------------*/
   wxGetApp().M_boo_mini_display_caption.toggle() ;
   ToggleWindowStyle( wxCAPTION ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::set_fullscreen( e_fullscreen_level fsl )
{
   /*----------------------------------------------------------------------*/
   if( wxGetApp().M_boo_mini_display.get() || m_fullscreen_level == fsl )
   {  return ; }

   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media_active = get_media_active() ;
   if( p_media_active == nullptr ) { return ; }

   /*----------------------------------------------------------------------*/
   bool boo_set_fullscreen = ( fsl != FSL_NONE ) ;
   bool boo_change_fullscreen = (    boo_set_fullscreen
                                  != ( m_fullscreen_level != FSL_NONE )
                                ) ;
   /*----------------------------------------------------------------------*/
   m_fullscreen_level = fsl ;

   /*--( Go to fullscreen => save and suppress panes )---------------------*/
   if( boo_change_fullscreen )
   {
      /*-------------------------------------------------------------------*/
      if( boo_set_fullscreen )
      {  m_s_before_fullscreen_perspective = m_auimgr.SavePerspective() ; }
      else
      {  m_auimgr.LoadPerspective( m_s_before_fullscreen_perspective ) ; }
      /*-------------------------------------------------------------------*/
      m_auimgr.GetPane( m_bopl ).Show( !boo_set_fullscreen ) ;
      m_auimgr.GetPane( get_window_pl() ).Show( !boo_set_fullscreen ) ;
      m_auimgr.GetPane( m_panel_ctrl ).Show( !boo_set_fullscreen ) ;
      /*-------------------------------------------------------------------*/
      ShowFullScreen( boo_set_fullscreen ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   layout_media() ;

   /*--( Back from fullscreen under MSW7, some refreshes have to be made )-*/
   #ifdef __WXMSW__
      m_panel_media->Refresh() ;
   #endif

   /*--( In fullscreen the cursor disappears but reappears on move )-------*/
   if( boo_change_fullscreen )
   {
      /*-------------------------------------------------------------------*/
      if( boo_set_fullscreen )
      {  /*----------------------------------------------------------------*/
         p_media_active->SetCursor( wxCursor( wxCURSOR_BLANK ) ) ;
         p_media_active->Bind( wxEVT_MOTION,
                               &CFrame::OnFullScreenMouseOn, this
                             ) ;
         /*----------------------------------------------------------------*/
      }
      else
      {  /*--( Unbind doesn't seem to work well under GTK )----------------*/
         p_media_active->SetCursor( wxCursor( wxNullCursor ) ) ;
         p_media_active->Unbind( wxEVT_MOTION,
                                 &CFrame::OnFullScreenMouseOn, this
                               ) ;
         /*----------------------------------------------------------------*/
      }
      /*--( Refresh title useless in fullscreen mode )---------------------*/
      start_viewer_timer( !boo_set_fullscreen ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   m_timer_mouse_off.Stop() ;
   init_gui_fullscreen() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::next_fullscreen()
{
   /*--( What will be the next "fullscreen" ? )----------------------------*/
   switch( m_fullscreen_level )
   {  /*-------------------------------------------------------------------*/
      case FSL_NONE :
         set_fullscreen( m_vec_media.size() > 1
                         ? FSL_ALL_MEDIA : FSL_ONE_MEDIA
                       ) ;
         break ;
      case FSL_ALL_MEDIA : set_fullscreen( FSL_ONE_MEDIA ) ; break ;
      case FSL_ONE_MEDIA : set_fullscreen( FSL_NONE )      ; break ;
      case FSL_NB : break ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::add_play_hist( const CChannel *p_channel )
{
   /*----------------------------------------------------------------------*/
   if( p_channel == nullptr ) { return ; }
   /*--( Change place of an already registered channel )-------------------*/
   auto it = std::find( m_ds_play_hist.begin(), m_ds_play_hist.end(),
                        p_channel->get_name()
                      ) ;
   if( it != m_ds_play_hist.end() )
   {  m_ds_play_hist.erase( it ) ; }
   /*--( Limit the history size to what is planned )-----------------------*/
   if( m_ds_play_hist.size() == mtf::co_i_play_hist_size - 1 )
   {  m_ds_play_hist.pop_back() ; }
   /*----------------------------------------------------------------------*/
   m_ds_play_hist.push_front( p_channel->get_name() ) ;
   /*--( Now the menu has to be updated )----------------------------------*/
   refresh_menu_play_hist() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::take_snapshot()
{
   /*----------------------------------------------------------------------*/
   CVlcMedia *p_media_active = get_media_active() ;
   if( p_media_active == nullptr ) { return ; }
   /*--( Default directory is the executable one ? )-----------------------*/
   if(    wxGetApp().M_s_snapshot_path.empty()
       || !wxDir::Exists( wxGetApp().M_s_snapshot_path.get() )
     )
   {  p_media_active->take_snapshot( wxGetApp().get_exe_path() ) ; }
   else
   {  p_media_active->take_snapshot( wxGetApp().M_s_snapshot_path.get() ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::check_version( int i_err_code, const wxString &s_data,
                            bool boo_check_manual
                          )
{
   /*-----------------------------------------------------------------------+
   ! Even if the last version data could not be retrieved the check is      !
   ! considered as done                                                     !
   +-----------------------------------------------------------------------*/
   wxGetApp().M_s_chkupd_last.set( wxDateTime::Today().FormatISODate() ) ;

   /*--( The two delimiters will handle CRLF & LF separators )-------------*/
   wxStringTokenizer tkz( s_data, "\r\n" ) ;
   wxString s_last_version ;
   wxString s_message ;
   /*----------------------------------------------------------------------*/
   if( tkz.HasMoreTokens() ) { s_last_version = tkz.GetNextToken() ; }

   /*--( Error ? )---------------------------------------------------------*/
   if(    i_err_code != CDownloadBase::DE_NO_ERROR_DONE
       || s_last_version.empty()
     )
   {  /*-------------------------------------------------------------------*/
      if( boo_check_manual )
      {  s_message.Printf( _( "%s's last version data could not be "
                              "retrieved"
                            ),
                           MTF_DISPLAY_NAME
                         ) ;
         wxMessageBox( s_message ) ;
      }
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Same old ? )------------------------------------------------------*/
   if( MTF_VERSION == s_last_version )
   {
      /*-------------------------------------------------------------------*/
      if( boo_check_manual )
      {  /*----------------------------------------------------------------*/
         s_message = _( "This version is the latest available" ) ;
         CDlgLastVersion( this, s_message, s_data ).get_modal() ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*--( New version ! )---------------------------------------------------*/
   s_message.Printf( _( "The version %s is available" ), s_last_version ) ;
   CDlgNewVersion( this, s_message, s_data ).get_modal() ;
   /*--( After new version installed the program is restarted )------------*/
   if( !wxGetApp().get_program_to_start_on_exit().empty() )
   {  Close( true ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CFrame::get_play_hist()
{
   /*----------------------------------------------------------------------*/
   wxString s_play_hist ;
   /*----------------------------------------------------------------------*/
   for( const auto &hist : m_ds_play_hist )
   {
      /*--------------------------------------------------------------------+
      ! Channel names are separated with lf because they can contain spaces !
      +--------------------------------------------------------------------*/
      if( !s_play_hist.empty() ) { s_play_hist += '\n' ; }
      /*-------------------------------------------------------------------*/
      s_play_hist += hist ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( s_play_hist ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CFrame::set_play_hist( const wxString &s_play_hist )
{
   /*----------------------------------------------------------------------*/
   m_ds_play_hist.clear() ;
   /*----------------------------------------------------------------------*/
   wxStringTokenizer tkz( s_play_hist, '\n' ) ;
   /*----------------------------------------------------------------------*/
   while(    tkz.HasMoreTokens()
          && m_ds_play_hist.size() < mtf::co_i_play_hist_size
        )
   {  m_ds_play_hist.emplace_back( tkz.GetNextToken() ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxDEFINE_EVENT( EVT_SET_TITLE, wxCommandEvent ) ;

/*-------------------------------------------------------------------------*/
class CThreadViewers : public wxThread
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      explicit CThreadViewers( CChannel *p_channel )
                             : m_p_channel( p_channel  )
      {   ; }
      //~CThreadGetViewers() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      CChannel *m_p_channel ;

   /*----------------------------------------------------------------------*/
   public :
      /*--( Will be executed by "Run" )------------------------------------*/
      virtual wxThread::ExitCode Entry() override
      {
         /*----------------------------------------------------------------*/
         if( m_p_channel->refresh_nb_viewers() == 0 )
         {  /*-------------------------------------------------------------*/
            wxQueueEvent( wxGetApp().m_frame,
                          new wxCommandEvent( EVT_SET_TITLE )
                        ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
         return( ( wxThread::ExitCode )0 ) ;
         /*----------------------------------------------------------------*/
      }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
void CFrame::refresh_nb_viewers( CChannel *p_channel )
{
   /*----------------------------------------------------------------------*/
   CThreadViewers *p_thread = new CThreadViewers( p_channel ) ;
   p_thread->Run() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                          End of file CFrame.cpp                           !
+==========================================================================*/
