/*====================================+=====================================+
! File CVlcInterface.cpp              ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/tokenzr.h>
#include <wx/uri.h>
#include "CVlcInterface.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
int const CVlcInterface::m_st_co_i_version_min = 2 ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CVlcInterface::CVlcInterface( const wxString &s_vlc_path,
                              const CChannel *p_channel
                            )
{
   /*----------------------------------------------------------------------*/
   wxString s_path = s_vlc_path ;
   /*----------------------------------------------------------------------*/
   #ifdef __WXMSW__
      /*--( By default the path to vlc executable is used )----------------*/
      if( s_path.empty() )
      {  s_path = wxGetApp().m_fn_vlc.GetPath() ; }
      /*-------------------------------------------------------------------*/
      {  /*--( Suppress useless warnings )---------------------------------*/
         wxLogNull no_log ;
         /*--( Revert to standard search )---------------------------------*/
         ::SetDllDirectory( NULL ) ;
         /*--( Set directory to load "sub" dlls loaded )-------------------*/
         ::SetDllDirectory( s_path ) ;
         /*--( Force path else if already loaded it won't work )-----------*/
         Load( wxFileName( s_path, "libvlc.dll" ).GetFullPath() ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   #elif defined( __WXGTK__ )
      /*--( libvlc can have different names ... )--------------------------*/
      wxString tb_s_libvlc[] = { "libvlc.so", "libvlc.so.5" } ;
      /*-------------------------------------------------------------------*/
      {
         /*----------------------------------------------------------------*/
         wxLogNull no_log ; // Suppress useless warnings
         /*----------------------------------------------------------------*/
         for( size_t sz_num = 0 ;
             sz_num < WXSIZEOF( tb_s_libvlc )
          && !Load( wxFileName( s_path, tb_s_libvlc[ sz_num ] ).GetFullPath()
                   ) ;
              ++sz_num
            )
         {  ; }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   #else
      #error Unmanaged platform to: load libvlc
   #endif
   /*----------------------------------------------------------------------*/
   if( !IsLoaded() )
   {  return ; }
   /*----------------------------------------------------------------------*/
   {  wxLogNull no_log ; // Suppress useless warnings
      /*-------------------------------------------------------------------*/
      INIT_P_FNC( libvlc_new ) ;
      INIT_P_FNC( VLC_Version ) ;
      INIT_P_FNC( libvlc_get_version ) ;
      INIT_P_FNC( libvlc_set_user_agent ) ;
      INIT_P_FNC( libvlc_set_app_id ) ;
      INIT_P_FNC( libvlc_media_player_new ) ;
      INIT_P_FNC( libvlc_media_player_event_manager ) ;
      INIT_P_FNC( libvlc_media_event_manager ) ;
      INIT_P_FNC( libvlc_media_add_option ) ;
      INIT_P_FNC( libvlc_event_attach ) ;
      INIT_P_FNC( libvlc_event_detach ) ;
      INIT_P_FNC( libvlc_event_type_name ) ;
      INIT_P_FNC( libvlc_media_player_release ) ;
      INIT_P_FNC( libvlc_release ) ;
      INIT_P_FNC( libvlc_media_player_get_xwindow ) ;
      INIT_P_FNC( libvlc_media_player_set_xwindow ) ;
      INIT_P_FNC( libvlc_media_player_get_hwnd ) ;
      INIT_P_FNC( libvlc_media_player_set_hwnd ) ;
      INIT_P_FNC( libvlc_media_new_location ) ;
      INIT_P_FNC( libvlc_media_new_path ) ;
      INIT_P_FNC( libvlc_media_player_get_media ) ;
      INIT_P_FNC( libvlc_media_player_set_media ) ;
      INIT_P_FNC( libvlc_video_get_track_description ) ;
      INIT_P_FNC( libvlc_video_get_track_count ) ;
      INIT_P_FNC( libvlc_video_get_track ) ;
      INIT_P_FNC( libvlc_video_set_track ) ;
      INIT_P_FNC( libvlc_audio_get_track_description ) ;
      INIT_P_FNC( libvlc_audio_get_track_count ) ;
      INIT_P_FNC( libvlc_audio_get_track ) ;
      INIT_P_FNC( libvlc_audio_set_track ) ;
      INIT_P_FNC( libvlc_video_get_spu_description ) ;
      INIT_P_FNC( libvlc_video_get_spu_count ) ;
      INIT_P_FNC( libvlc_track_description_list_release ) ;
      INIT_P_FNC( libvlc_video_get_spu ) ;
      INIT_P_FNC( libvlc_video_set_spu ) ;
      INIT_P_FNC( libvlc_media_release ) ;
      INIT_P_FNC( libvlc_media_player_play ) ;
      INIT_P_FNC( libvlc_media_player_pause ) ;
      INIT_P_FNC( libvlc_media_player_stop ) ;
      INIT_P_FNC( libvlc_media_player_is_playing ) ;
      INIT_P_FNC( libvlc_media_player_get_state ) ;
      INIT_P_FNC( libvlc_media_get_stats ) ;
      INIT_P_FNC( libvlc_errmsg ) ;
      INIT_P_FNC( libvlc_audio_get_volume ) ;
      INIT_P_FNC( libvlc_audio_set_volume ) ;
      INIT_P_FNC( libvlc_audio_get_mute ) ;
      INIT_P_FNC( libvlc_audio_set_mute ) ;
      INIT_P_FNC( libvlc_get_fullscreen ) ;
      INIT_P_FNC( libvlc_set_fullscreen ) ;
      INIT_P_FNC( libvlc_video_take_snapshot ) ;
      INIT_P_FNC( libvlc_video_set_mouse_input ) ;
      INIT_P_FNC( libvlc_video_set_deinterlace ) ;
      INIT_P_FNC( libvlc_video_set_key_input ) ;
      INIT_P_FNC( libvlc_set_exit_handler ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   if( !init_version() || !is_version_min_ok() )
   {  return ; }

   /*--( Some parameters to pass to "libvlc" ? )---------------------------*/
   std::vector< const char * > vec_argv ;
   if( p_channel != nullptr )
   {
      /*-------------------------------------------------------------------*/
      wxString s_init = build_init_string( p_channel ) ;
      /*-------------------------------------------------------------------*/
      wxStringTokenizer stok_argv( s_init, " " ) ;
      while( stok_argv.HasMoreTokens() )
      {  vec_argv.push_back( wxStrdup( stok_argv.GetNextToken() ) ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   const char **p_p_c_argv = ( vec_argv.empty() ? nullptr : &vec_argv[ 0 ] );
   m_inst = m_libvlc_new( vec_argv.size(), p_p_c_argv ) ;

   /*--( The allocated data have to be deleted )---------------------------*/
   for( const char *p_c : vec_argv ) { delete p_c ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CVlcInterface::~CVlcInterface()
{
   /*----------------------------------------------------------------------*/
   if( m_inst != nullptr ) { m_libvlc_release( m_inst ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
bool CVlcInterface::init_version()
{
   /*----------------------------------------------------------------------*/
   const char *p_c_version = get_version() ;
   /*----------------------------------------------------------------------*/
   if(    p_c_version == nullptr
       || wxSscanf( p_c_version, "%d.", &m_i_version ) != 1
     )
   {  return( false ) ; }
   /*----------------------------------------------------------------------*/
   return( true ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
const char *CVlcInterface::get_version() const
{
   /*----------------------------------------------------------------------*/
   return( m_libvlc_get_version != nullptr
           ? m_libvlc_get_version()
           : ( m_VLC_Version != nullptr ? m_VLC_Version() : nullptr )
         ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CVlcInterface::st_build_init_string( const CChannel *p_channel,
                                              int i_vlc_version
                                            )
{
   /*----------------------------------------------------------------------*/
   if( !wxGetApp().get_use_vlc_init_str() ) { return( wxEmptyString ) ; }
   /*----------------------------------------------------------------------*/
   wxString s_init = wxGetApp().M_s_vlc_init_all.get() ;
   if( p_channel != nullptr )
   {
      /*-------------------------------------------------------------------*/
      {  wxString s_init_channel = p_channel->get_vlc_init() ;
         /*----------------------------------------------------------------*/
         if( !s_init_channel.empty() )
         {  if( !s_init.empty() ) { s_init += ' ' ; }
            s_init += s_init_channel ;
         }
         /*----------------------------------------------------------------*/
      }

      /*--------------------------------------------------------------------+
      ! For youtube channels a param is added/removed depending on          !
      ! VLC version                                                         !
      +--------------------------------------------------------------------*/
      if( p_channel->get_pl_known() == PK_LIVE_YT )
      {
         /*-----------------------------------------------------------------+
         ! Why this strange word beginning/ending regular expression ?      !
         ! Because the "\y", "\m", "\M" anchors don't seem to work with the !
         ! first two '-' of the searched string                             !
         +-----------------------------------------------------------------*/
         const wxString co_s_yt_vlc2 = "--demux=avformat" ;
         const wxString co_s_re = "(?:^|\\s)" + co_s_yt_vlc2 + "(?:$|\\s)" ;
         bool boo_present = ( mtf::match( s_init, co_s_re ) > 0 ) ;
         /*----------------------------------------------------------------*/
         if( boo_present && i_vlc_version >= 3 )
         {  s_init.Replace( co_s_yt_vlc2, wxEmptyString ) ; }
         else
         if( !boo_present && i_vlc_version <= 2 )
         {  if( !s_init.empty() ) { s_init += ' ' ; }
            s_init += co_s_yt_vlc2 ;
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }

   /*--( Adding the VLC proxy options )------------------------------------*/
   if( !wxGetApp().M_s_proxy_server.empty() )
   {
      /*-------------------------------------------------------------------*/
      wxURI uri = wxGetApp().M_s_proxy_server.get() ;
      /*-------------------------------------------------------------------*/
      if( !s_init.empty() ) { s_init += ' ' ; }
      /*-------------------------------------------------------------------*/
      if( !uri.GetScheme().StartsWith( "socks" ) )
      {
         /*--( The scheme is added to the command line option )------------*/
         s_init += "--http-proxy=" + wxGetApp().M_s_proxy_server.get() ;
         /*----------------------------------------------------------------*/
      }
      else
      {  /*--( The scheme is not added to the command line option )--------*/
         s_init += "--socks=" + uri.GetServer() ;
         if( uri.HasPort() ) { s_init += ':' + uri.GetPort() ; }
         /*----------------------------------------------------------------*/
         if( uri.HasUserInfo() )
         {
            /*-------------------------------------------------------------*/
            wxString s_value = uri.GetUser() ;
            if( !s_value.empty() )
            {  s_init += " --socks-user=" + s_value ; }
            /*-------------------------------------------------------------*/
            s_value = uri.GetPassword() ;
            if( !s_value.empty() )
            {  s_init += " --socks-pwd=" + s_value ; }
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( s_init ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                       End of file CVlcInterface.cpp                       !
+==========================================================================*/
