/*====================================+=====================================+
! File matelefree.cpp                 ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/clipbrd.h>
#include <wx/filename.h>
#include <wx/tokenzr.h>
#include <wx/txtstrm.h>
#include <wx/regex.h>
#include <curl/curl.h>
#include "matelefree.h"
#include "matelefree_info.h"
#include "CApp.h"
#include "CFrame.h"
#include "CDownload.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
wxMenuItem *mtf::insert_item( wxMenu *menu, int i_pos, int i_id,
                              const wxString &s_item,
                              const wxBitmap &bmp,
                              const wxString &s_help,
                              wxItemKind mik,
                              wxMenu *p_submenu
                            )
{
   /*----------------------------------------------------------------------*/
   wxMenuItem *mi
            = new wxMenuItem( menu, i_id, s_item, s_help, mik, p_submenu ) ;
   /*----------------------------------------------------------------------*/
   if( bmp.IsOk() ) { mi->SetBitmap( bmp ) ; }
   /*----------------------------------------------------------------------*/
   menu->Insert( i_pos, mi ) ;
   /*----------------------------------------------------------------------*/
   return( mi ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxMenuItem *mtf::append_item( wxMenu *menu, int i_id,
                              const wxString &s_item,
                              const wxBitmap &bmp,
                              const wxString &s_help,
                              wxItemKind mik,
                              wxMenu *p_submenu
                            )
{
   /*----------------------------------------------------------------------*/
   wxMenuItem *mi
            = new wxMenuItem( menu, i_id, s_item, s_help, mik, p_submenu ) ;
   /*----------------------------------------------------------------------*/
   if( bmp.IsOk() ) { mi->SetBitmap( bmp ) ; }
   /*----------------------------------------------------------------------*/
   menu->Append( mi ) ;
   /*----------------------------------------------------------------------*/
   return( mi ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxBitmap mtf::get_menu_bmp( const wxArtID &id )
{
   /*----------------------------------------------------------------------*/
   return( wxArtProvider::GetBitmap( id, wxART_MENU, wxSize( 16, 16 ) ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxBitmap mtf::get_toolbar_bmp( const wxArtID &id )
{
   /*----------------------------------------------------------------------*/
   return( wxArtProvider::GetBitmap( id, wxART_TOOLBAR, wxSize( 32, 32 ) ) );
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxMenuItem *mtf::append_item( wxMenu *menu, int i_id, const wxArtID &artid )
{
   /*----------------------------------------------------------------------*/
   return( mtf::append_item( menu, i_id,
                             wxGetStockLabel( i_id,  wxSTOCK_WITH_MNEMONIC
                                                   | wxSTOCK_WITH_ACCELERATOR
                                            ),
                             mtf::get_menu_bmp( artid )
                           )
         ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxMenuItem *mtf::append_item( wxMenu *dst, wxMenuItem *mi_src )
{
   /*----------------------------------------------------------------------*/
   wxMenu *menu_sub = nullptr ;
   if( mi_src->IsSubMenu() )
   {  menu_sub = new wxMenu ;
      mtf::append_menu( menu_sub, mi_src->GetSubMenu() ) ;
   }
   /*----------------------------------------------------------------------*/
     wxMenuItem *mi_dst
   = mtf::append_item( dst, mi_src->GetId(), mi_src->GetItemLabel(),
                       mi_src->GetBitmap(), mi_src->GetHelp(),
                       mi_src->GetKind(), menu_sub
                     ) ;
   /*----------------------------------------------------------------------*/
   mi_dst->Enable( mi_src->IsEnabled() ) ;
   /*----------------------------------------------------------------------*/
   if( mi_src->IsCheck() || mi_src->IsRadio() )
   {  mi_dst->Check( mi_src->IsChecked() ) ; }
   /*----------------------------------------------------------------------*/
   return( mi_dst ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxMenuItem *mtf::append_item( wxMenu *dst, wxMenu *src, int i_id )
{
   /*----------------------------------------------------------------------*/
   wxMenuItem *mi = src->FindItem( i_id ) ;
   if( mi != nullptr )
   {  mi = mtf::append_item( dst, mi ) ; }
   /*----------------------------------------------------------------------*/
   return( mi ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxMenuItem *mtf::append_item( wxMenu *dst, wxMenuBar *src, int i_id )
{
   /*----------------------------------------------------------------------*/
   wxMenuItem *mi = src->FindItem( i_id ) ;
   if( mi != nullptr )
   {  mi = mtf::append_item( dst, mi ) ; }
   /*----------------------------------------------------------------------*/
   return( mi ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::append_menu( wxMenu *dst, wxMenu *src )
{
   /*----------------------------------------------------------------------*/
   for( const auto &mi_src : src->GetMenuItems() )
   {  append_item( dst, mi_src ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::wxs2int( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   long l_val = 0 ;
   /*----------------------------------------------------------------------*/
   s.ToLong( &l_val ) ;
   return( l_val ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::rand( int i_min, int i_max )
{
   /*----------------------------------------------------------------------*/
   if( i_min >= i_max ) { return( i_min ) ; }
   /*----------------------------------------------------------------------*/
   return( i_min + std::rand() % ( i_max - i_min + 1 ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::set_clipboard( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   if( wxTheClipboard->Open() )
   {  /*-------------------------------------------------------------------*/
      wxTheClipboard->SetData( new wxTextDataObject( s ) ) ;
      wxTheClipboard->Close() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::unspace( wxString s )
{
   /*----------------------------------------------------------------------*/
   int i_pos ;
   while( ( i_pos = s.find( ' ' ) ) >= 0 ) { s.erase( i_pos, 1 ) ; }
   /*----------------------------------------------------------------------*/
   return( s ) ;
   /*----------------------------------------------------------------------*/
}

/*--( "Computation" of the non accented version of a character )-----------*/
static wxUniChar st_unaccent( const wxUniChar &c )
{
   /*--( Ascii means non accented ... )------------------------------------*/
   if( c.IsAscii() ) { return( c ) ; }
   /*----------------------------------------------------------------------*/
   switch( c.GetValue() )
   {
      /*-------------------------------------------------------------------*/
      case 0x01E2 : case 0x01FC :
         return( 0x00C6 ) ;
      case 0x00C0 : case 0x00C1 : case 0x00C2 : case 0x00C3 : case 0x00C4 :
      case 0x00C5 : case 0x0100 : case 0x0102 : case 0x0104 : case 0x01CD :
      case 0x01DE : case 0x01E0 : case 0x01FA : case 0x0200 : case 0x0202 :
      case 0x0226 :
         return( 'A' ) ;
      case 0x00C7 : case 0x0106 : case 0x0108 : case 0x010A : case 0x010C :
      case 0x0187 :
         return( 'C' ) ;
      case 0x010E : case 0x0110 :
         return( 'D' ) ;
      case 0x00C8 : case 0x00C9 : case 0x00CA : case 0x00CB : case 0x0112 :
      case 0x0114 : case 0x0116 : case 0x0118 : case 0x011A : case 0x0204 :
      case 0x0206 : case 0x0228 :
         return( 'E' ) ;
      case 0x011C : case 0x011E : case 0x0120 : case 0x0122 : case 0x01E4 :
      case 0x01E6 : case 0x01F4 :
         return( 'G' ) ;
      case 0x0124 : case 0x0126 : case 0x021E :
         return( 'H' ) ;
      case 0x00CC : case 0x00CD : case 0x00CE : case 0x00CF : case 0x0128 :
      case 0x012A : case 0x012C : case 0x012E : case 0x0130 : case 0x01CF :
      case 0x0208 : case 0x020A :
         return( 'I' ) ;
      case 0x0134 :
         return( 'J' ) ;
      case 0x0136 : case 0x01E8 :
         return( 'K' ) ;
      case 0x0139 : case 0x013B : case 0x013D : case 0x013F : case 0x0141 :
         return( 'L' ) ;
      case 0x00D1 : case 0x0143 : case 0x0145 : case 0x0147 : case 0x014A :
      case 0x01F8 :
         return( 'N' ) ;
      case 0x00D2 : case 0x00D3 : case 0x00D4 : case 0x00D5 : case 0x00D6 :
      case 0x014C : case 0x014E : case 0x0150 : case 0x01A0 : case 0x01D1 :
      case 0x01FE : case 0x020C : case 0x020E : case 0x022A : case 0x022C :
      case 0x022E : case 0x0230 :
         return( 'O' ) ;
      case 0x01EA : case 0x01EC :
         return( 'Q' ) ;
      case 0x0154 : case 0x0156 : case 0x0158 :
         return( 'R' ) ;
      case 0x008A : case 0x015A : case 0x015C : case 0x015E : case 0x0160 :
         return( 'S' ) ;
      case 0x0162 : case 0x0164 : case 0x0166 : case 0x021A :
         return( 'T' ) ;
      case 0x00D9 : case 0x00DA : case 0x00DB : case 0x00DC : case 0x0168 :
      case 0x016A : case 0x016C : case 0x016E : case 0x0170 : case 0x0172 :
      case 0x01AF : case 0x01D3 : case 0x01D5 : case 0x01D7 : case 0x01D9 :
      case 0x01DB : case 0x0214 : case 0x0216 :
         return( 'U' ) ;
      case 0x00DD : case 0x009F : case 0x0176 : case 0x0178 : case 0x0232 :
         return( 'Y' ) ;
      case 0x0174 :
         return( 'W' ) ;
      case 0x008E : case 0x0179 : case 0x017B : case 0x017D : case 0x0224 :
         return( 'Z' ) ;
      case 0x01E3 : case 0x01FD :
         return( 0x00E6 ) ;
      case 0x00E0 : case 0x00E1 : case 0x00E2 : case 0x00E3 : case 0x00E4 :
      case 0x00E5 : case 0x0101 : case 0x0103 : case 0x0105 : case 0x01CE :
      case 0x01DF : case 0x01E1 : case 0x01FB : case 0x0201 : case 0x0203 :
      case 0x0227 :
         return( 'a' ) ;
      case 0x00E7 : case 0x0107 : case 0x0109 : case 0x010B : case 0x010D :
      case 0x0188 :
         return( 'c' ) ;
      case 0x010F : case 0x0111 :
         return( 'd' ) ;
      case 0x00E8 : case 0x00E9 : case 0x00EA : case 0x00EB : case 0x0113 :
      case 0x0115 : case 0x0117 : case 0x0119 : case 0x011B : case 0x0205 :
      case 0x0207 : case 0x0229 :
         return( 'e' ) ;
      case 0x011D : case 0x011F : case 0x0121 : case 0x0123 : case 0x01E5 :
      case 0x01E7 : case 0x01F5 :
         return( 'g' ) ;
      case 0x0125 : case 0x0127 : case 0x021F :
         return( 'h' ) ;
      case 0x00EC : case 0x00ED : case 0x00EE : case 0x00EF : case 0x0129 :
      case 0x012B : case 0x012D : case 0x012F : case 0x01D0 : case 0x0209 :
      case 0x020B :
         return( 'i' ) ;
      case 0x0135 : case 0x01F0 :
         return( 'j' ) ;
      case 0x0137 : case 0x0138 : case 0x01E9 :
         return( 'k' ) ;
      case 0x013A : case 0x013C : case 0x013E : case 0x0140 : case 0x0142 :
         return( 'l' ) ;
      case 0x00F1 : case 0x0144 : case 0x0146 : case 0x0148 : case 0x0149 :
      case 0x014B : case 0x01F9 :
         return( 'n' ) ;
      case 0x00F2 : case 0x00F3 : case 0x00F4 : case 0x00F5 : case 0x00F6 :
      case 0x014D : case 0x014F : case 0x0151 : case 0x01A1 : case 0x01D2 :
      case 0x01FF : case 0x020D : case 0x020F : case 0x022B : case 0x022D :
      case 0x022F : case 0x0231 :
         return( 'o' ) ;
      case 0x01EB : case 0x01ED :
         return( 'q' ) ;
      case 0x0155 : case 0x0157 : case 0x0159 :
         return( 'r' ) ;
      case 0x009A : case 0x015B : case 0x015D : case 0x015F : case 0x0161 :
         return( 's' ) ;
      case 0x0163 : case 0x0165 : case 0x0167 : case 0x021B :
         return( 't' ) ;
      case 0x00F9 : case 0x00FA : case 0x00FB : case 0x00FC : case 0x0169 :
      case 0x016B : case 0x016D : case 0x016F : case 0x0171 : case 0x0173 :
      case 0x01B0 : case 0x01D4 : case 0x01D6 : case 0x01D8 : case 0x01DA :
      case 0x01DC : case 0x0215 : case 0x0217 :
         return( 'u' ) ;
      case 0x00FD : case 0x00FF : case 0x0177 : case 0x0233 :
         return( 'y' ) ;
      case 0x0175 :
         return( 'w' ) ;
      case 0x009E : case 0x017A : case 0x017C : case 0x017E : case 0x0225 :
         return( 'z' ) ;
      /*-------------------------------------------------------------------*/
   } ;

   /*----------------------------------------------------------------------*/
   return( c ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::unaccent( wxString s )
{
   /*----------------------------------------------------------------------*/
   const int co_i_nb = static_cast< int >( s.size() ) ;
   /*----------------------------------------------------------------------*/
   for( int i_num = 0 ; i_num < co_i_nb ; ++i_num )
   {  s[ i_num ] = st_unaccent( s[ i_num ] ) ; }
   /*----------------------------------------------------------------------*/
   return( s ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::prepare_string( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   wxString s_prep = mtf::unspace( mtf::unaccent( s ).Lower() ) ;
   /*--( Convert largest list of non authorized file name characters )-----*/
   int i_pos = 0 ;
   while( ( i_pos = s_prep.find_first_of( R"=(\/:*?"<>|)=", i_pos ) ) >= 0 )
   {  s_prep[ i_pos ] = '-' ; }
   /*----------------------------------------------------------------------*/
   return( s_prep ) ;
   /*----------------------------------------------------------------------*/
}

/*--( Basic html chars conversion )----------------------------------------*/
wxString mtf::unhtml( const wxString &s_in )
{
   /*----------------------------------------------------------------------*/
   wxString s_out ;
   s_out.reserve( s_in.size() ) ;
   /*----------------------------------------------------------------------*/
   static mtf::t_map_strint st_map_char_seq
   = { { "AElig"   ,  198 }, { "Aacute"  ,  193 }, { "Acirc"   ,  194 },
       { "Agrave"  ,  192 }, { "Alpha"   ,  913 }, { "Aring"   ,  197 },
       { "Atilde"  ,  195 }, { "Auml"    ,  196 }, { "Beta"    ,  914 },
       { "Ccedil"  ,  199 }, { "Chi"     ,  935 }, { "Dagger"  , 8225 },
       { "Delta"   ,  916 }, { "ETH"     ,  208 }, { "Eacute"  ,  201 },
       { "Ecirc"   ,  202 }, { "Egrave"  ,  200 }, { "Epsilon" ,  917 },
       { "Eta"     ,  919 }, { "Euml"    ,  203 }, { "Gamma"   ,  915 },
       { "Iacute"  ,  205 }, { "Icirc"   ,  206 }, { "Igrave"  ,  204 },
       { "Iota"    ,  921 }, { "Iuml"    ,  207 }, { "Kappa"   ,  922 },
       { "Lambda"  ,  923 }, { "Mu"      ,  924 }, { "Ntilde"  ,  209 },
       { "Nu"      ,  925 }, { "OElig"   ,  338 }, { "Oacute"  ,  211 },
       { "Ocirc"   ,  212 }, { "Ograve"  ,  210 }, { "Omega"   ,  937 },
       { "Omicron" ,  927 }, { "Oslash"  ,  216 }, { "Otilde"  ,  213 },
       { "Ouml"    ,  214 }, { "Phi"     ,  934 }, { "Pi"      ,  928 },
       { "Prime"   , 8243 }, { "Psi"     ,  936 }, { "Rho"     ,  929 },
       { "Scaron"  ,  352 }, { "Sigma"   ,  931 }, { "THORN"   ,  222 },
       { "Tau"     ,  932 }, { "Theta"   ,  920 }, { "Uacute"  ,  218 },
       { "Ucirc"   ,  219 }, { "Ugrave"  ,  217 }, { "Upsilon" ,  933 },
       { "Uuml"    ,  220 }, { "Xi"      ,  926 }, { "Yacute"  ,  221 },
       { "Yuml"    ,  376 }, { "Zeta"    ,  918 }, { "aacute"  ,  225 },
       { "acirc"   ,  226 }, { "acute"   ,  180 }, { "aelig"   ,  230 },
       { "agrave"  ,  224 }, { "alefsym" , 8501 }, { "alpha"   ,  945 },
       { "amp"     ,   38 }, { "and"     , 8743 }, { "ang"     , 8736 },
       { "apos"    ,   39 }, { "aring"   ,  229 }, { "asymp"   , 8776 },
       { "atilde"  ,  227 }, { "auml"    ,  228 }, { "bdquo"   , 8222 },
       { "beta"    ,  946 }, { "brvbar"  ,  166 }, { "bull"    , 8226 },
       { "cap"     , 8745 }, { "ccedil"  ,  231 }, { "cedil"   ,  184 },
       { "cent"    ,  162 }, { "chi"     ,  967 }, { "circ"    ,  710 },
       { "clubs"   , 9827 }, { "cong"    , 8773 }, { "copy"    ,  169 },
       { "crarr"   , 8629 }, { "cup"     , 8746 }, { "curren"  ,  164 },
       { "dArr"    , 8659 }, { "dagger"  , 8224 }, { "darr"    , 8595 },
       { "deg"     ,  176 }, { "delta"   ,  948 }, { "diams"   , 9830 },
       { "divide"  ,  247 }, { "eacute"  ,  233 }, { "ecirc"   ,  234 },
       { "egrave"  ,  232 }, { "empty"   , 8709 }, { "emsp"    , 8195 },
       { "ensp"    , 8194 }, { "epsilon" ,  949 }, { "equiv"   , 8801 },
       { "eta"     ,  951 }, { "eth"     ,  240 }, { "euml"    ,  235 },
       { "euro"    , 8364 }, { "exist"   , 8707 }, { "fnof"    ,  402 },
       { "forall"  , 8704 }, { "frac12"  ,  189 }, { "frac14"  ,  188 },
       { "frac34"  ,  190 }, { "frasl"   , 8260 }, { "gamma"   ,  947 },
       { "ge"      , 8805 }, { "gt"      ,   62 }, { "hArr"    , 8660 },
       { "harr"    , 8596 }, { "hearts"  , 9829 }, { "hellip"  , 8230 },
       { "iacute"  ,  237 }, { "icirc"   ,  238 }, { "iexcl"   ,  161 },
       { "igrave"  ,  236 }, { "image"   , 8465 }, { "infin"   , 8734 },
       { "int"     , 8747 }, { "iota"    ,  953 }, { "iquest"  ,  191 },
       { "isin"    , 8712 }, { "iuml"    ,  239 }, { "kappa"   ,  954 },
       { "lArr"    , 8656 }, { "lambda"  ,  955 }, { "lang"    , 9001 },
       { "laquo"   ,  171 }, { "larr"    , 8592 }, { "lceil"   , 8968 },
       { "ldquo"   , 8220 }, { "le"      , 8804 }, { "lfloor"  , 8970 },
       { "lowast"  , 8727 }, { "loz"     , 9674 }, { "lrm"     , 8206 },
       { "lsaquo"  , 8249 }, { "lsquo"   , 8216 }, { "lt"      ,   60 },
       { "macr"    ,  175 }, { "mdash"   , 8212 }, { "micro"   ,  181 },
       { "middot"  ,  183 }, { "minus"   , 8722 }, { "mu"      ,  956 },
       { "nabla"   , 8711 }, { "nbsp"    ,  160 }, { "ndash"   , 8211 },
       { "ne"      , 8800 }, { "ni"      , 8715 }, { "not"     ,  172 },
       { "notin"   , 8713 }, { "nsub"    , 8836 }, { "ntilde"  ,  241 },
       { "nu"      ,  957 }, { "oacute"  ,  243 }, { "ocirc"   ,  244 },
       { "oelig"   ,  339 }, { "ograve"  ,  242 }, { "oline"   , 8254 },
       { "omega"   ,  969 }, { "omicron" ,  959 }, { "oplus"   , 8853 },
       { "or"      , 8744 }, { "ordf"    ,  170 }, { "ordm"    ,  186 },
       { "oslash"  ,  248 }, { "otilde"  ,  245 }, { "otimes"  , 8855 },
       { "ouml"    ,  246 }, { "para"    ,  182 }, { "part"    , 8706 },
       { "permil"  , 8240 }, { "perp"    , 8869 }, { "phi"     ,  966 },
       { "pi"      ,  960 }, { "piv"     ,  982 }, { "plusmn"  ,  177 },
       { "pound"   ,  163 }, { "prime"   , 8242 }, { "prod"    , 8719 },
       { "prop"    , 8733 }, { "psi"     ,  968 }, { "quot"    ,   34 },
       { "rArr"    , 8658 }, { "radic"   , 8730 }, { "rang"    , 9002 },
       { "raquo"   ,  187 }, { "rarr"    , 8594 }, { "rceil"   , 8969 },
       { "rdquo"   , 8221 }, { "real"    , 8476 }, { "reg"     ,  174 },
       { "rfloor"  , 8971 }, { "rho"     ,  961 }, { "rlm"     , 8207 },
       { "rsaquo"  , 8250 }, { "rsquo"   , 8217 }, { "sbquo"   , 8218 },
       { "scaron"  ,  353 }, { "sdot"    , 8901 }, { "sect"    ,  167 },
       { "shy"     ,  173 }, { "sigma"   ,  963 }, { "sigmaf"  ,  962 },
       { "sim"     , 8764 }, { "spades"  , 9824 }, { "sub"     , 8834 },
       { "sube"    , 8838 }, { "sum"     , 8721 }, { "sup"     , 8835 },
       { "sup1"    ,  185 }, { "sup2"    ,  178 }, { "sup3"    ,  179 },
       { "supe"    , 8839 }, { "szlig"   ,  223 }, { "tau"     ,  964 },
       { "there4"  , 8756 }, { "theta"   ,  952 }, { "thetasym",  977 },
       { "thinsp"  , 8201 }, { "thorn"   ,  254 }, { "tilde"   ,  732 },
       { "times"   ,  215 }, { "trade"   , 8482 }, { "uArr"    , 8657 },
       { "uacute"  ,  250 }, { "uarr"    , 8593 }, { "ucirc"   ,  251 },
       { "ugrave"  ,  249 }, { "uml"     ,  168 }, { "upsih"   ,  978 },
       { "upsilon" ,  965 }, { "uuml"    ,  252 }, { "weierp"  , 8472 },
       { "xi"      ,  958 }, { "yacute"  ,  253 }, { "yen"     ,  165 },
       { "yuml"    ,  255 }, { "zeta"    ,  950 }, { "zwj"     , 8205 },
       { "zwnj"    , 8204 }
     } ;

   /*----------------------------------------------------------------------*/
   for( auto cit = s_in.begin() ; cit != s_in.end() ; ++cit )
   {
      /*--( Not a html symbol )--------------------------------------------*/
      if( *cit != '&' || cit + 1 == s_in.end() || cit + 2 == s_in.end() )
      {  s_out += *cit ; }
      else /*--( A numeric sequence : "&#[x|X]{.};" )----------------------*/
      if(  *( cit + 1 ) == '#' )
      {  /*----------------------------------------------------------------*/
         cit += 2 ;
         /*--( Base used to specify the character code )-------------------*/
         int i_base ;
         if( *cit == 'x' || *cit == 'X' )
         {  i_base = 16 ; ++cit ; }
         else
         {  i_base = 10 ; }
         /*----------------------------------------------------------------*/
         wxString s_num ;
         while( cit != s_in.end() && *cit != ';' ) { s_num += *cit++ ; }
         /*----------------------------------------------------------------*/
         long l_char ;
         s_num.ToLong( &l_char, i_base ) ;
         s_out += wxUniChar( l_char ) ;
         /*----------------------------------------------------------------*/
      }
      else
      {  /*--( Extract next html symbol : between '&' and ';' )------------*/
         wxString s_seq ;
         auto cit_seq = cit + 1 ;
         while( cit_seq != s_in.end() && wxIsalpha( *cit_seq ) )
         {  s_seq += *cit_seq++ ; }
         /*--( Normally a sequence ends with a ';' => non alpha )----------*/
         if( cit_seq == s_in.end() || *cit_seq != ';' )
         {  s_out += *cit ; }
         else
         {  /*--( Search for known html sequence )-------------------------*/
            auto it_map = st_map_char_seq.find( s_seq ) ;
            /*--( Replace if found )---------------------------------------*/
            if( it_map == st_map_char_seq.end() )
            {  s_out.append( cit, cit_seq + 1 ) ; }
            else
            {  s_out += it_map->second ; }
            /*-------------------------------------------------------------*/
            cit = cit_seq ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( s_out ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::url_encode( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   const wxScopedCharBuffer scb_s( s.utf8_str() ) ;
   wxString s_res ;
   s_res.reserve( s.size() * 2 ) ;
   /*----------------------------------------------------------------------*/
   const size_t co_sz_length = scb_s.length() ;
   for( size_t sz_pos = 0 ; sz_pos != co_sz_length ; ++sz_pos )
   {
      /*-------------------------------------------------------------------*/
      const wxUint8 u8_c = scb_s.data()[ sz_pos ] ;
      /*--( Byte to encode ? )---------------------------------------------*/
      if(    ( u8_c >= '0' && u8_c <= '9' )
          || ( u8_c >= 'A' && u8_c <= 'Z' )
          || ( u8_c >= 'a' && u8_c <= 'z' )
          || u8_c == '-'
          || u8_c == '_'
          || u8_c == '.'
          || u8_c == '~'
        )
      {  s_res += u8_c ; }
      else
      {  /*----------------------------------------------------------------*/
         static const char st_tb_c_hexdigit[] =
         { '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F' };
         /*----------------------------------------------------------------*/
         s_res += '%' ;
         s_res += st_tb_c_hexdigit[ ( u8_c >> 4 ) & 0x0F ] ;
         s_res += st_tb_c_hexdigit[ u8_c & 0x0F ] ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( s_res ) ;
   /*----------------------------------------------------------------------*/
}

/*--( Check if a string is ok to be an "ini" entry name )------------------*/
int mtf::check_ini_entry_name( const wxString &s_name, wxString &s_err_msg )
{
   /*--( Path separators are seen as such by wx )--------------------------*/
   const char *co_p_c_forbidden = R"=(/\)=" ;
   if( s_name.find_first_of( co_p_c_forbidden ) != wxString::npos )
   {  /*-------------------------------------------------------------------*/
      s_err_msg.Printf( _( "Characters \"%s\" are not allowed" ),
                        co_p_c_forbidden
                      ) ;
      return( -1 ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::tlw_set_rect( wxWindow *p_win, const wxRect &rect )
{
   /*----------------------------------------------------------------------*/
   if( rect.GetSize() == wxDefaultSize )
   {  p_win->SetSize( p_win->GetMinSize() ) ; }
   else
   {  p_win->SetSize( rect.GetSize() ) ; }
   /*----------------------------------------------------------------------*/
   if( rect.GetPosition() == wxDefaultPosition )
   {  p_win->CenterOnParent() ; }
   else
   {  p_win->SetPosition( rect.GetPosition() ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::init_tlw_rect( wxWindow *p_win, const CConfigRect &cfg_rect )
{
   /*----------------------------------------------------------------------*/
   p_win->SetMinSize( p_win->GetEffectiveMinSize() ) ;
   /*----------------------------------------------------------------------*/
   mtf::tlw_set_rect( p_win, cfg_rect.get() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::save_tlw_rect( wxWindow *p_win, CConfigRect &cfg_rect )
{
   /*----------------------------------------------------------------------*/
   cfg_rect.set( p_win->GetRect() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::draw_wrapped( wxDC &dc, const wxString &s )
{
   /*----------------------------------------------------------------------*/
   const int co_i_margin = 8 ;
   wxPoint pt_write( co_i_margin, co_i_margin ) ;
   const int co_i_max_width = dc.GetSize().GetWidth() - 2 * co_i_margin ;
   /*----------------------------------------------------------------------*/
   wxSize size_token ;
   wxString s_current ;
   wxString_cit cit_end = s.begin() ;
   bool boo_fit ;
   size_t sz_len_ok ;
   const int co_i_max_smaller = 3 ;
   int i_smaller = 0 ;
   /*----------------------------------------------------------------------*/
   while( cit_end != s.end() && i_smaller < co_i_max_smaller )
   {
      /*-------------------------------------------------------------------*/
        const int co_i_max_height
      = dc.GetSize().GetHeight() - pt_write.y - co_i_margin ;
      /*-------------------------------------------------------------------*/
      do
      {  /*----------------------------------------------------------------*/
         sz_len_ok = s_current.size() ;
         /*--( Add another word )------------------------------------------*/
         for( ; cit_end != s.end() && *cit_end != ' ' && *cit_end != '-' ;
              ++cit_end
            )
         {  s_current += *cit_end ; }
         /*----------------------------------------------------------------*/
         size_token = dc.GetTextExtent( s_current ) ;
         boo_fit = (    size_token.GetWidth() <= co_i_max_width
                     && size_token.GetHeight() <= co_i_max_height
                   ) ;
         /*--( As it doesn't fit, last word is removed )-------------------*/
         if( !boo_fit && sz_len_ok > 0 )
         {  cit_end -= s_current.size() - sz_len_ok ;
            s_current.Remove( sz_len_ok ) ;
         }
         /*--( Add seperator after first word )----------------------------*/
         if( boo_fit && cit_end != s.end() )
         {  s_current += *cit_end++ ; }
         /*----------------------------------------------------------------*/
      } while( boo_fit && cit_end != s.end() ) ;

      /*-------------------------------------------------------------------*/
      if( !boo_fit && sz_len_ok == 0 && i_smaller < co_i_max_smaller )
      {
         /*-----------------------------------------------------------------+
         ! Test smaller font if only one word doesn't fit                   !
         ! The font used will remain "smaller" because it seems visually    !
         ! nicer to look at                                                 !
         +-----------------------------------------------------------------*/
         bool boo_ok = false ;
         /*----------------------------------------------------------------*/
         for( ; !boo_ok && i_smaller < co_i_max_smaller ; ++i_smaller )
         {  /*-------------------------------------------------------------*/
            dc.SetFont( dc.GetFont().Smaller() ) ;
            /*-------------------------------------------------------------*/
            size_token = dc.GetTextExtent( s_current ) ;
            boo_ok = (    size_token.GetWidth()  <= co_i_max_width
                       && size_token.GetHeight() <= co_i_max_height
                     ) ;
            /*-------------------------------------------------------------*/
         }
         /*----------------------------------------------------------------*/
      }

      /*-------------------------------------------------------------------*/
      dc.DrawText( s_current, pt_write ) ;
      pt_write.y += size_token.GetHeight() ;
      /*-------------------------------------------------------------------*/
      s_current.clear() ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
#ifdef __WXMSW__
/*-------------------------------------------------------------------------*/
#ifndef MTF_MAKEFOURCC
#define MTF_MAKEFOURCC(ch0,ch1,ch2,ch3) \
  ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) | \
   ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24))
#endif /* MTF_MAKEFOURCC */
/*-------------------------------------------------------------------------*/
int mtf::msw_exe_bitness( const wxString &s_filename )
{
   /*----------------------------------------------------------------------*/
   wxLogNull no_log ; // Suppress useless warnings
   wxFile   fi( s_filename ) ;
   wxUint32 dw_offset ;
   wxUint32 dw_sign ;
   IMAGE_FILE_HEADER ifh ;

   /*--( Access the right data )-------------------------------------------*/
   if(    !fi.IsOpened()
       || fi.Seek( 0x3c ) == wxInvalidOffset
       || fi.Read( &dw_offset, sizeof( dw_offset ) ) == wxInvalidOffset
       || fi.Seek( dw_offset ) == wxInvalidOffset
       || fi.Read( &dw_sign, sizeof( dw_sign ) ) == wxInvalidOffset
       || dw_sign != MTF_MAKEFOURCC( 'P','E','\0','\0' )
       || fi.Read( &ifh, sizeof( ifh ) ) ==  wxInvalidOffset
     )
   {  return( -1 ) ; }

   /*----------------------------------------------------------------------*/
   switch( ifh.Machine )
   {  /*-------------------------------------------------------------------*/
      case IMAGE_FILE_MACHINE_I386  : return( 32 ) ;
      /*-------------------------------------------------------------------*/
      case IMAGE_FILE_MACHINE_IA64  :
      case IMAGE_FILE_MACHINE_AMD64 : return( 64 ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( -2 ) ;
   /*----------------------------------------------------------------------*/
}
#endif

/*-------------------------------------------------------------------------*/
wxString mtf::read_m3u_line( wxTextInputStream &tis, int &i_line_num )
{
   /*----------------------------------------------------------------------*/
   wxString s ;
   /*--( Empty and comment lines are skipped )-----------------------------*/
   while(    !tis.GetInputStream().Eof()
          && (    ( s = tis.ReadLine() ).Trim().empty()
               || s.StartsWith( "#EXTREM:" )
             )
        )
   {  ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   if( !tis.GetInputStream().Eof() ) { ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   return( s ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::read_line( wxTextInputStream &tis, int &i_line_num )
{
   /*----------------------------------------------------------------------*/
   wxString s ;
   /*--( Empty and comment lines are skipped )-----------------------------*/
   while(    !tis.GetInputStream().Eof()
          && (    ( s = tis.ReadLine() ).Trim().empty()
               || s.StartsWith( "#" )
             )
        )
   {  ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   if( !tis.GetInputStream().Eof() ) { ++i_line_num ; }
   /*----------------------------------------------------------------------*/
   return( s ) ;
   /*----------------------------------------------------------------------*/
}

/*--------------------------------------------------------------------------+
! All parameters after "s_re" are supposed to be "wxString *"               !
! representing destination of the different sub-expression in the regex.    !
! For example : match( "toto", ".(.).(.)", &s1, &s2 ) ;                     !
!                                                                           !
! If a param has to be skipped, pass nullptr as its parameter.              !
! For example : match( "toto", ".(.).(.)", nullptr, &s2 ) ;                 !
!                                                                           !
! By default the "ignore case" is set !                                     !
!                                                                           !
! The return value is : GetMatchCount()                                     !
+--------------------------------------------------------------------------*/
int mtf::match( const wxString &s_data, const wxString &s_re ... )
{
   /*----------------------------------------------------------------------*/
   wxRegEx re( s_re, wxRE_ADVANCED | wxRE_ICASE ) ;
   /*----------------------------------------------------------------------*/
   if( !re.IsValid() ) { return( -1 ) ; }
   if( !re.Matches( s_data ) ) { return( -2 ) ; }

   /*----------------------------------------------------------------------*/
   va_list args ;
   va_start( args, s_re ) ;
   /*--( 1 param for the match of the whole expression )-------------------*/
   const int co_i_nb_param = re.GetMatchCount() ;
   for( int i_num_param = 1 ; i_num_param < co_i_nb_param ; ++i_num_param )
   {  /*-------------------------------------------------------------------*/
      wxString *p_s = va_arg( args, wxString * ) ;
      if( p_s != nullptr ) { *p_s = re.GetMatch( s_data, i_num_param ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   va_end( args ) ;
   /*----------------------------------------------------------------------*/
   return( co_i_nb_param ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
static size_t st_libcurl_header_callback( char *buffer, size_t size,
                                          size_t nitems, void *userdata
                                        )
{
   /*----------------------------------------------------------------------*/
   wxString *p_s_url = static_cast< wxString * >( userdata ) ;
   const size_t co_sz_size = nitems * size ;
   const wxString co_s_data( buffer, co_sz_size ) ;
   /*----------------------------------------------------------------------*/
   mtf::match( co_s_data, R"=(\mlocation: (.*)\M)=", p_s_url ) ;
   /*----------------------------------------------------------------------*/
   return( co_sz_size ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::urlencode( const wxString &s_in )
{
   /*----------------------------------------------------------------------*/
   wxString s_out ;
   /*----------------------------------------------------------------------*/
   for( auto c : s_in )
   {  s_out += (    wxIsalpha( c ) || wxIsdigit( c )
                 || c == '-' || c == '.' || c == '_' || c == '~'
                 ? c : wxString::Format( "%%%02X", c )
               ) ;
   }
   /*----------------------------------------------------------------------*/
   return( s_out ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::unshort_url( wxString &s_url, wxString *p_s_err_msg )
{
   /*----------------------------------------------------------------------*/
   CURL *curl = curl_easy_init() ;
   int i_ret = 0 ;
   /*----------------------------------------------------------------------*/
   if( curl == nullptr ) { return( -1 ) ; }
   /*----------------------------------------------------------------------*/
   wxCharBuffer cb_url = s_url.ToAscii() ;
   curl_easy_setopt( curl, CURLOPT_URL, cb_url.data() ) ;
   CDownload::st_init_proxy( curl ) ;
   curl_easy_setopt( curl, CURLOPT_HEADER, 1 ) ;
   curl_easy_setopt( curl, CURLOPT_HEADERDATA, &s_url ) ;
   curl_easy_setopt( curl, CURLOPT_HEADERFUNCTION,
                     st_libcurl_header_callback
                   ) ;
   curl_easy_setopt( curl, CURLOPT_SSL_VERIFYPEER, 0 ) ;
   curl_easy_setopt( curl, CURLOPT_FOLLOWLOCATION, 0 ) ;

   /*-----------------------------------------------------------------------+
   ! GO ! GO ! GO !                                                         !
   ! Note that the "write error" error can happen for "periscope" ...       !
   +-----------------------------------------------------------------------*/
   CDownloadBase::e_err_code err_code
         = CDownloadBase::st_curle_to_de( curl, curl_easy_perform( curl ) ) ;
   if(    err_code != CDownloadBase::DE_NO_ERROR_DONE
       && err_code != CDownloadBase::DE_WRITE_ERROR
     )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = CDownloadBase::st_get_err_msg( err_code ) ; }
      /*-------------------------------------------------------------------*/
      i_ret = -2 ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   curl_easy_cleanup( curl ) ;
   /*----------------------------------------------------------------------*/
   return( i_ret ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::download( const wxString &s_url, wxString &s_data,
                   wxString *p_s_last_url, int i_connect_timeout,
                   const mtf::t_lst_string &lst_header,
                   const wxString &s_post
                 )
{
   /*----------------------------------------------------------------------*/
   CDownload dl( s_url, s_data ) ;
   /*----------------------------------------------------------------------*/
   CDownloadBase *p_dlbase = &dl.get_dlbase() ;
   p_dlbase->set_connect_timeout( i_connect_timeout ) ;
   p_dlbase->set_header( lst_header ) ;
   p_dlbase->set_post( s_post ) ;
   int i_ret = dl.non_threaded_exec() ;
   /*----------------------------------------------------------------------*/
   if( i_ret == CURLE_OK && p_s_last_url != nullptr )
   {  *p_s_last_url = dl.get_last_url() ; }
   /*----------------------------------------------------------------------*/
   return( i_ret ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int mtf::download( const wxString &s_url, wxMemoryOutputStream &mos,
                   wxString *p_s_last_url, int i_connect_timeout,
                   const mtf::t_lst_string &lst_header,
                   const wxString &s_post
                 )
{
   /*----------------------------------------------------------------------*/
   CDownload dl( s_url, mos ) ;
   /*----------------------------------------------------------------------*/
   CDownloadBase *p_dlbase = &dl.get_dlbase() ;
   p_dlbase->set_connect_timeout( i_connect_timeout ) ;
   p_dlbase->set_header( lst_header ) ;
   p_dlbase->set_post( s_post ) ;
   int i_ret = dl.non_threaded_exec() ;
   /*----------------------------------------------------------------------*/
   if( i_ret == CURLE_OK && p_s_last_url != nullptr )
   {  *p_s_last_url = dl.get_last_url() ; }
   /*----------------------------------------------------------------------*/
   return( i_ret ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxColor mtf::alt_bg_color( wxColor color_bg )
{
   /*--( Cf "wxListCtrlBase::EnableAlternateRowColours" wxWidgets code )---*/
   int i_alpha = ( color_bg.GetRGB() > 0x808080 ? 97 : 150 ) ;
   return( color_bg.ChangeLightness( i_alpha ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString mtf::get_debug_filename()
{
   /*----------------------------------------------------------------------*/
   return( wxFileName( wxGetApp().get_exe_path(), MTF_NAME "_debug.txt"
                     ).GetFullPath()
         ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void mtf::trace( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   wxFile f( mtf::get_debug_filename(), wxFile::write_append ) ;
   /*----------------------------------------------------------------------*/
   f.Write( wxDateTime::UNow().Format( "%H:%M:%S.%l : " ) + s + "\n" ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
size_t CConv_UTF8_8859_1::ToWChar( wchar_t *dst, size_t dstLen,
                                   const char *src, size_t srcLen
                                 ) const
{
   /*----------------------------------------------------------------------*/
   size_t sz_ret_conv = wxConvUTF8.ToWChar( dst, dstLen, src, srcLen ) ;
   /*-----------------------------------------------------------------------+
   ! If the last conversion failed and the available "temporary" space is   !
   ! filled (seems to be 4 bytes) the 8859_1 conversion is done.            !
   +-----------------------------------------------------------------------*/
   if( sz_ret_conv == wxCONV_FAILED && srcLen >= dstLen - 1 )
   {  return( wxConvISO8859_1.ToWChar( dst, dstLen, src, srcLen ) ) ; }
   /*----------------------------------------------------------------------*/
   return( sz_ret_conv ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgProgress::CDlgProgress( wxWindow *parent )
            : wxDialog( parent, wxID_ANY, wxEmptyString )
{
   /*--( At least under GTK this dialog's icon is not set. It is forced )--*/
   SetIcon( wxGetApp().m_frame->GetIcon() ) ;
   m_bsz_gen = new wxBoxSizer( wxVERTICAL ) ;
   /*----------------------------------------------------------------------*/
     m_stx_msg
   = new wxStaticText( this, wxID_ANY, wxEmptyString,
                       wxDefaultPosition, wxDefaultSize,
                       wxALIGN_CENTRE | wxST_NO_AUTORESIZE
                     ) ;
   m_bsz_gen->Add( m_stx_msg, 1, wxALL | wxEXPAND, 10 ) ;
   /*----------------------------------------------------------------------*/
   m_ga_progress = new wxGauge( this, wxID_ANY, 100 ) ;
   m_ga_progress->SetValue( 0 ) ;
   m_ga_progress->SetMinSize( wxSize( 500, -1 ) ) ;
   m_bsz_gen->Add( m_ga_progress, 1, wxBOTTOM|wxEXPAND|wxLEFT|wxRIGHT, 10 ) ;
   /*----------------------------------------------------------------------*/
   m_bt_quit = new wxButton( this, wxID_CANCEL ) ;
   m_bsz_gen->Add( m_bt_quit, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5 ) ;
   m_bt_quit->SetFocus() ;
   m_bt_quit->SetDefault() ;

   /*----------------------------------------------------------------------*/
   SetSizer( m_bsz_gen ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
   m_bt_quit->Bind( wxEVT_COMMAND_BUTTON_CLICKED, &CDlgProgress::OnBtCancel,
                    this
                  ) ;
   /*----------------------------------------------------------------------*/
}

/*--------------------------------------------------------------------------+
! Menu :                                                                    !
! The accelerator key modifiers "Shift", "Ctrl" and "Alt" must not be       !
! translated. They are "automatically" translated by the menuing system     !
! if the following translations are given.                                  !
+--------------------------------------------------------------------------*/
#define DUMMY_TRANSLATIONS wxTRANSLATE( "ctrl" )  \
                           wxTRANSLATE( "shift" ) \
                           wxTRANSLATE( "alt" )
#undef DUMMY_TRANSLATIONS

/*--( The program entry point ! )------------------------------------------*/
IMPLEMENT_APP( CApp )
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file matelefree.cpp                         !
+==========================================================================*/
